; Lab 4 (2002)  Motor Control and Hardware Timer
; author: Ken Clowes
; date:  November 19, 2002
;
;  *** CHANGES FROM OFFICIAL LAB  ***
;   1) Part I uses SOFTWARE DELAYS to do:
;      a) Initialization turns off both motors
;      b) After 2 seconds, both motors turn on in FWD direction
;      c) After an additional 5 seconds (7 seconds total),
;         the Starboard motor reverses direction.
;      d) After an additional 3 seconds, both motors turn off
;
;   2) Part II loops until either bumper is activated.
;
;   3) Part III repeats Part I but uses TOF interrupts
;
;   LCD Display Format
;
;                               1 1 1 1 1 1(1 1 1 1) 
;     col   0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5(6 7 8 9)
;     line1 P o r t :   *   S t a r :   *
;     line2 P A R T ?
;
;        Where:
;                * = 'F' (On-FWD), 'R' (On-REV) or '-' (OFF)
;                ? ='1' OR '2' or '3'


       org $6000
main:
       jsr init

forever:
       jsr doLab4
       jmp forever

init:
       sei
       jsr clearLCD
       jsr starboardOFF
       jsr portOFF
       jsr alarmsInit
       jsr displayLCD
       rts

doLab4:
       jsr displayLCD
       jsr doPart1
       jsr displayLCD
       jsr doPart2
       jsr displayLCD
       jsr doPart3
    
       ;stop       ;Comment-out this instruction for continuous operation
       rts

doPart1:
       ldaa #'1
       staa LAB4PartNumber
       jsr displayLCD
       ;wait 2 sec, then turn on both motors
       ldx #2000
       jsr delayXms
       jsr starboardON_FWD
       jsr portON_FWD
       ldaa #'F
       staa msgPortState
       staa msgStarState
       jsr displayLCD

       ;wait 5 secs, then starboard reverse
       ldx #5000
       jsr delayXms
       jsr starboardON_REV      
       ldaa #'R
       staa msgStarState
       jsr displayLCD

       ;wait 3 secs, then both motors OFF
       ldx #3000
       jsr delayXms
       jsr starboardOFF
       jsr portOFF
       ldaa #'-
       staa msgPortState
       staa msgStarState
       jsr displayLCD
       rts

doPart2:
       ldaa #'2
       staa LAB4PartNumber
       ldaa #' '
       staa LAB4PartNumber+1
       jsr displayLCD
       ldx #lab2HitToContinueMessage
       jsr putstr

; Wait until at least one bumper is hit
doPart2_loop:
      jsr checkBumpers
      cmpd #0
      beq doPart2_loop
      ldx #lab2HitMessage
      jsr putstr
      rts


doPart3:
       clr LAB4PartNumber+1
       ldaa #'3
       staa LAB4PartNumber
       jsr clearLCD
       jsr displayLCD
      ;set up TOFalarm for about 3 seconds (15*3 TOF ticks)
       ldaa #15*3
       jsr setTOFalarm

      ;Enable interrupts
       cli

      ;Loop until alarm is fired
P3Wait1:
       jsr checkTOFalarm
       bpl P3Wait1

      ;Alarm has fired, so turn on motors
       ldaa #'F
       staa msgPortState
       staa msgStarState
       jsr displayLCD
       jsr starboardON_FWD
       jsr portON_FWD

      ;set up TOFalarm for about 2 seconds (2*15 TOF ticks)
       ldaa #2*15
       jsr setTOFalarm

      ;Loop until alarm is fired
       jsr waitTOFalarm

      ;Alarm has fired, so turn on both motors REV
       ldaa #'R
       staa msgPortState
       staa msgStarState
       jsr displayLCD
       jsr starboardON_REV
       jsr portON_REV

      ;set up TOFalarm for about 4 seconds (4*15 TOF ticks)
       ldaa #4*15
       jsr setTOFalarm

      ;Loop until alarm is fired
       jsr waitTOFalarm

      ;Alarm has fired, so starboard ON-FWD
       ldaa #'F
       staa msgStarState
       jsr starboardON_FWD
       jsr displayLCD

      ;set up TOFalarm for about 5 seconds (5*15 TOF ticks)
       ldaa #5*15
       jsr setTOFalarm

      ;Loop until alarm is fired
       jsr waitTOFalarm

      ;Alarm has fired, so port OFF
       ldaa #'-
       staa msgPortState
       jsr portOFF
       jsr displayLCD

      ;set up TOFalarm for about 2 seconds (2*15 TOF ticks)
       ldaa #2*15
       jsr setTOFalarm

      ;Loop until alarm is fired
       jsr waitTOFalarm

      ;Alarm has fired, so turn off motors
       ldaa #'-
       staa msgPortState
       staa msgStarState
       jsr displayLCD
       jsr starboardOFF
       jsr portOFF

       rts

displayLCD:
       jsr homeLCD
       ldx #message
       jsr putstrLCD
       rts

waitTOFalarm
       jsr checkTOFalarm
       bpl waitTOFalarm
       rts


message:
       fcc "Port: "
msgPortState:
       fcb '?'
       fcc " Star: "
msgStarState:
       fcb '?'

;Terminate line
       fcb NEWLINE

       fcc "PART "
LAB4PartNumber:
       fcb 'x'
       fcb 0

continueMsg:
       fcc "Hit bumper..."
       fcb 0


lab2HitToContinueMessage fcc "Hit a bumper to continue..."
               fcb NEWLINE
               fcb 0

lab2HitMessage fcc "Bumper hit--continung to Part 3"
               fcb NEWLINE
               fcb 0

#include "motors.asm"
#include "bumpers.asm"
#include "TOFalarms.asm"
#include "stdlib.asm"
#include "eebot.equ"
#include "6811.equ"
