;;  @library stdlib
;;  A collection of useful subroutines.
;;   <code>ble</code>
;;
;; another par...			
#include "6811.equ"

;*******Following equates come from BUFFALO Monitor
        
INCHAR           equ $E55B      ;Pointer to INputCHARacter routine

;; @name doMenu
;; Displays choices on the terminal and gets user selection.
;; There can be up to nine (9) choices identifiable by a single
;; digit number.  The choices are displayed and the user then
;; enters one of the valid numbers.  A binary number between 1  
;; and N_CHOICES-1 is returned.
;;
;; <center><b>End-User interface</b></center>
;;
;; The user must enter a selection (by hitting a key and then hitting the 
;; ENTER (or RETURN) key.  Nothing happens until the ENTER key is hit.
;; If the selection is valid, the calling routine recieves a valid number 
;; in AccA.  If the user chose an invalid selection (eg. if only selections 
;; 1, 2 and 3 were valid and the user typed "4" [or any other invalid
;; selection], then an error message is displayed to the end-user and
;; they are prompted to "try again".
;;
;; <center><b>Programmer interface</b></center>
;;
;; If the number of choices specified is not in the range 1..9, the
;; results are unpredictable.  <em>However</em>, the current implementation
;; causes a fatal error:  a message is printed and an SWI instruction
;; is invoked.
;; 
;; @param IX Starting address of menu string.
;; @param AccA number of <em>choices</em> (must be >= 1 and <= 9)
;; @return AccA a valid selection number
;; @side CC modified.
;; @example
;; <pre>
;;        ldx #myMenu
;;        ldaa #MENU_N_CHOICES
;;        jsr doMenu
;;        swi
;;
;;MENU_N_CHOICES equ 2
;;myMenu  fcc "    Choose one of the following"
;;        fcb NEWLINE
;;        fcc "  1: Do Timer Alarms"
;;        fcb NEWLINE
;;        fcc "  2: Do Virtual Robot"     
;;        fcb NEWLINE, 0
;; </pre>	

;; @name putstrGEN
;; Outputs a string to any device.  The device could be the LCD display,
;; the EEPROM subsystem, the terminal connected to the serial port, or
;; anything else.
;;
;; @param IX Starting address of string.
;; @param IY Address of subroutine to output a single character
;; @return Nothing
;; @side None
putstrGEN: 
        psha   ;prologue---save registers
        tpa
        psha
        pshx
pstG_loop:       
        ldaa 0,x
        beq pstG_done
        jsr 0,y
        inx
        bra pstG_loop
pstG_done:
        pulx   ;epilogue---restore registers
        pula
        tap
        pula
        rts

;; @name putstr
;; Outputs a string to the terminal.
;;
;; @param IX Starting address of string.
;; @return Nothing
;; @side None
putstr: 
	psha   ;prologue---save registers
	tpa
	psha
	pshy
	
	ldy #putchar
	jsr putstrGEN

	puly  ;epilogue---restore registers
	pula
	tap
	pula
	rts

;; @name putchar
;; Outputs a single character to the terminal.
;;
;; @param  AccA--ascii code of character to print
;; @return Nothing
;; @side NOT KNOWN (Implementation is simply an adaptor to Buffalo
;;      Monitor funciton.)
putchar:        
       tst SCSR         ;read status
       bpl putchar
       anda #$7F        ;mask parity
       staa SCDAT       ;send character
       rts

;; @name getchar
;; Gets a single character from the keyboard.  This input method is
;; <b>not</b> interrupt-driven; it waits until the user types a key.  The
;; key is echoed.       
;;
;; @param none
;; @return  AccA--ascii code of character typed.
;; @side NOT KNOWN (Implementation is simply an adaptor to Buffalo
;;      Monitor funciton.)
getchar:
        jsr INCHAR
        rts
        

hexAsciiTable fcc '0123456789ABCDEF'  ;used by "itoaHex_u8"

;; @name itoaHex_u8
;; Converts a byte to its Ascii Hex representation.
;;
;; @param AccA the byte to convert
;; @return AccA Char representation of most sigificant nibble
;; @return AccB Char representation of least sigificant nibble
;; @side CC modified
;; @example
;; Consider:
;; <pre>
;;     ldaa #58       ;58 (decimal) same as 0x3a
;;     jsr itoaHex_u8
;;       ;On return AccA  is '3' and AccB is 'A'
;; </pre>
;;
;; <b>Notes:</b> Library routine developed in Lab 2.
;;
itoaHex_u8
           ;prologue---save registers
        pshx

        ldx #hexAsciiTable
        psha
        anda #$0f
        tab
        abx
        ldaa 0,x
        pulb
        psha

        ldx #hexAsciiTable
        lsrb
        lsrb
        lsrb
        lsrb
        abx
        ldaa 0,x
        pulb

           ;epilogue---restore registers
        pulx
        rts

;; @name clearLCD
;; Clears the LCD display and sets the cursor to home.
;; 
;; @return nothing
;; @side CC modified
;;

clearLCD
        tst LCD_STATUS          ; Wait till not busy.
        bmi clearLCD
        psha
        ldaa #LCD_CLEAR_CMD
        staa LCD_CONTROL
        pula
        rts

;; @name homeLCD
;; Sets LCD cursor to home.  The display is <em>not</em> cleared.
;; 
;; @return nothing
;; @side CC modified
;;
;  BUG in my original version that did NOT have any observable effect
;  on my implementation of Lab 2.  I was lucky?  NO, I was unluckly...
;  The best way to detect and track down bugs is to have code that
;  does not work...  Alas, my program DID work....  I discovered the
;  bug by READING my own source code and recognizing that it was
;  "brain dead"...but would often work!  (Actually, "reading your own
;  code" or, preferably, having someone else read it, is an established
;  and well documented strategy for finding potential flaws in code.)
;
;  OK, what was the bug, why did it have no discernable effect and why
;  should it be fixed?
;
; When I first wrote the routine, I simply cut and pasted the already
; working "clearLCD" routine, changed its name and used the LCD_HOME_CMD
; obtaining:
;
;    homeLCD
;            tst LCD_STATUS          ; Wait till not busy.
;            bmi clearLCD
;            psha
;            ldaa #LCD_HOME_CMD
;            staa LCD_CONTROL
;            pula
;            rts
;
; Can you see the bug?  (Hint: look at the "bmi" instruction.)
; Can you see why the program could still work? (In my case, fixing the
; bug reduced LCD flicker by an almost imperceptible amount...)
; The bug is fixed in the version below.

homeLCD:
        tst LCD_STATUS          ; Wait till not busy.
        bmi homeLCD
        psha
        ldaa #LCD_HOME_CMD
        staa LCD_CONTROL
        pula
        rts

;; @name putcharLCD
;; Prints a character to the LCD display.  The character goes to the
;; next display position <b>unless</b> it is a newline character.  A
;; newline character moves the cursor to the beginning 
;; of the second line.
;; Results for unprintable characters are undefined; note the special
;; treatment of a newline character.
;; 
;; @param AccA ASCII code for character to display.
;; @return nothing
;; @side CC register modified.
;;
NEWLINE     equ 10
CARRIAGE_RETURN equ 13
LINE_TWO_LCD equ $C0
        
putcharLCD
        tst LCD_STATUS          ; Wait till not busy.
        bmi putcharLCD
        cmpa #NEWLINE
        beq setLine2LCD
        staa LCD_DATA   
        bra pcLCD_done
setLine2LCD
        psha
        ldaa #LINE_TWO_LCD
        staa LCD_CONTROL
        pula
pcLCD_done:     
        rts

;; @name putstrLCD
;; Prints a string to the LCD display.  If the string is longer
;; than the available space on the display, the results are undefined.
;;      
;; @param IX Starting address of null terminated string.
;; @return nothing
;; @side none
;;

putstrLCD:
	psha   ;prologue---save registers
	tpa
	psha
	pshy
	
	ldy #putcharLCD
	jsr putstrGEN

	puly  ;epilogue---restore registers
	pula
	tap
	pula
	rts


;; @name getAllADChannels
;; Samples and stores all 8 channels of the A/D subsystem.
;; IX must contain the address of an 8-byte buffer which will be
;; overwritten.  The time for the routine is less than 350 E-clock
;; cycles (i.e. less than 0.35ms on a 1 Mz system.)

;;      
;; @param IX the starting address of the raw results buffer (8 bytes)
;; @return nothing
;; @side CC modified
;;
        
getAllADChannels
                ;prologue---save registers
        psha
        pshx
        pshy
        
        ;Initiate read on channels 0--3
        ldaa #AD_DO_CH_0to3
        staa ADCTL
        ldy #AD_CHANNEL_0
        
        ;Wait for conversion complete
        jsr waitADDone
        
        ;Store results to buffer (IY = &measured; IX = &buffer)
        ldaa 0,y        
        staa 0,x

        ldaa 1,y        
        staa 1,x

        ldaa 2,y        
        staa 2,x

        ldaa 3,y        
        staa 3,x

        ;Initiate read on channels 4--7
        ldaa #AD_DO_CH_4to7
        staa ADCTL
        ldy #AD_CHANNEL_0
        
        ;Wait for conversion complete
        jsr waitADDone  
        
        ;Store results to buffer (IY = &measured; IX = &buffer[4])
        ldaa 0,y        
        staa 4,x
        
        ldaa 1,y        
        staa 5,x
        
        ldaa 2,y        
        staa 6,x
        
        ldaa 3,y        
        staa 7,x

        ;epilogue---restore registers
        puly
        pulx
        pula
        rts
        
; PRIVATE subroutine
; utility routine to wait until AD subsystem is NOT BUSY
waitADDone
        tst AD_STATUS
        bpl waitADDone
        rts

;; @name delay1ms
;; Software delay loop for 1000 bus cycles.  This corresponds to 1 ms
;; on a 4 MHz 6811 (E clock = 1 MHz).  If interrupts are enabled, the
;; actual time between subroutine invocation and return may be greater
;; than 1000 bus cycles.  If no interrupts occur, the delay will be
;; <B>exactly</B> 1000 cycles (including the time for invoking
;; the subroutine with a jsr instruction).
;;
;; @param none
;; @return nothing
;; @side none
;;

        ;Total delay = overhead + N*(2+3)
        ;            = 6+3+2+3+2+4+2+4+2+25 + 5N
        ;            ==> 5N = 1000-35 = 965
        ;            ==>  N = 193
DEL_COUNT      equ  193
        ; Assumed called with JSR EXT mode addressing ==> 6 cycles
delay1ms       psha             ; 3 cycles
               tpa              ; 2 cycles
               psha             ; 3 cycles
               ldaa #DEL_COUNT  ; 2 cycles
delay1msLoop   deca             ; 2 cycles
               bne delay1msLoop ; 3 cycles
               pula             ; 4 cycles
               tap              ; 2 cycles
               pula             ; 4 cycles
               nop              ; 2 cycles (PADDING)
               nop              ; 2 cycles (PADDING)
               rts              ; 5 cycles

;; @name delayXms
;; Software delay loop for about X ms.  (Actually,
;; 1000*X E-clock cycles; with a 1 MHz E-clock, this means X milliseconds.)
;; If interrupts are
;; enabled, the actual time between subroutine invocation and return
;; may be greater.  With no interrupts, the delay is a bit more than X
;; ms.  (For large X, the delay will be no more than 1% greater than
;; specificed.)
;;
;; <b>Note:</b> If X is zero, it does <em>not</em> mean a short delay!
;; It means delay by 65536 ms, ie. more than a minute.
;;
;; <b>Note:</b> For accurate time delays, use the
;; timer module (with or without interrupts.)
;;
;; @param IX number of clock cycles to delay
;; @return nothing
;; @side none
;;

; n_cycles = overhead + x*1006 = 27 + 1006*x
delayXms:
                  ;prologue---save registers
	       psha             ; 3 cycles
               tpa              ; 2 cycles
               psha             ; 3 cycles
	       pshx             ; ?4 cycles
DXM_Loop   jsr delay1ms     ; 1000 cycles
               dex          ; ?3 cycles
               bne DXM_Loop ; 3 cycles
                   ;epilogue---restore registers
               pulx             ; ?5 cycles
               pula             ; 4 cycles
               tap              ; 2 cycles
               pula             ; 4 cycles
               rts              ; 5 cycles

;; @name itoa_u16
;; Converts an unsigned 16-bit number to a decimal string.
;;
;; @param AccD 16-bit unsigned number to convert
;; @param IX Starting address of string.
;; @return Nothing
;; @side CC modified

itoa_u16:
	psha
	pshb
	pshy
	pshx
	pshx
	puly

	; zero is a special case
	cmpd #0
	bne u16_cont	
		ldaa #'0
		staa 0,x
		clr 1,x
		pulx
                bra u16_epilog

        ; it's not zero
u16_cont:
	ldx #10
	idiv			; AccB is remainder, IX is quotient
	addb #'0		; Convert remainder to ASCII
	stab 0,y
	iny
	cmpx #0
	beq u16_done
	xgdx
	bra u16_cont

u16_done:
	clr 0,y			; ensure generated string is null-terminated
	pulx
	jsr strrev		; string is in reverse order-->reverse it!
u16_epilog:
	puly			; restore original registers
	pulb
	pula
	rts

;; @name strrev
;; Reverses a string in place.  The IX register points to a null-terminated
;; string.  The same string is reversed in place (not copied).  If the
;; string is more than 255 characters in length, the results are
;; unpredictable.
;;
;; @param IX--starting address of the string to be reversed
;; @return nothing
;; @side Condition Code register (CC) modified.
;; @since 1.0
;; @author Ken Clowes
;; @example
;;  The string "abcd" will be reversed to the string "dcba".
;;

strrev:
	psha
	pshb
	pshy
	pshx
	pshx
	puly
	clrb
	; push chars onto stack
rev_push:	
	ldaa 0,x
		beq rev_all_pushed
		psha
		inx
		incb
	bra rev_push
	
rev_all_pushed:
	pshy			; Now, pop them back off
	pulx
	tstb
rev_pop_next:	
		beq rev_all_popped
		pula
		staa 0,x
		inx
		decb
	bra rev_pop_next
rev_all_popped:
	pulx
	puly
	pulb
	pula
	rts

;; @name itoa_u8
;; Converts an unsigned 8-bit number to a decimal string.
;;
;; @param AccA 8-bit unsigned number to convert
;; @param IX Starting address of string.
;; @return Nothing
;; @side CC modified

itoa_u8:
	psha
	pshb
	tab
	clra
	jsr itoa_u16
	pulb
	pula
	rts

;; @name itoa_s16
;; Converts a signed 16-bit number to a decimal string.
;;
;; @param AccD 16-bit signed number to convert
;; @param IX Starting address of string.
;; @return Nothing
;; @side CC modified

itoa_s16:
	pshx   ;prologue---save registers

	tsta
	bpl ITAS16_endif
	psha
	ldaa #'-
	staa 0,x
	inx
	pula
	coma  ;is there a "negd" instruction?
	comb
	addd #1

ITAS16_endif
	jsr itoa_u16

	pulx   ;epilogue---restore registers
	rts
