package com.kclowes.sim11;
public abstract class AbstractInstruction implements Instruction
{
    public abstract int  execute();
    CPU6811 cpu =  CPU6811.getInstance();
    private MemorySpace mem = MemorySpace64K.getInstance();
    final void push16(int p)
    {
	cpu.decrementSPby2();
	writeOperand16(p, (cpu.getSP()+1)&0xffff);
    }

    final int pop16()
    {
	cpu.incrementSPby2();
	return read16((cpu.getSP()-1)&0xffff);
    }

    final void push8(int p)
    {
	writeOperand8(p, cpu.getSP());
	cpu.decrementSP();
    }

    final int pop8()
    {
	cpu.incrementSP();
	return read8(cpu.getSP());
    }

    /** Push "all" registers.
     *
     * <p>
     * Pushes the registers PC, IY, IX, AccB, AccA, CCR in that order.
     * This corresponds to the order that registers are pushed onto
     * the stack when an interrupt is recognized.
     */

    public void pushAll()
    {
  	push16(cpu.getPC());
  	push16(cpu.getIY());
  	push16(cpu.getIX());
	push8(cpu.getAccA());
	push8(cpu.getAccB());
	push8(cpu.getCC());
	return;
    }

    /** Pop "all" registers.
     *
     * <p>
     * Pops the registers CCR, AccA, AccB, IX, IY, PC CC in that order.
     * This corresponds to the order that registers are pushed onto
     * the stack when an interrupt is recognized.
     */

    public void popAll()
    {
 	cpu.setCC(pop8());
 	cpu.setAccB(pop8());
	cpu.setAccA(pop8());
  	cpu.setIX(pop16());
  	cpu.setIY(pop16());
  	cpu.setPC(pop16());
	return;
    }

    final public int add8(int p, int q)
    {
	int result = ((p&0xff) + (q&0xff))&0xffff;
	setNZ8(result);
	cpu.CC_C = (result > 0xff);
	cpu.CC_V = (result > 0xff) != (((p&0x7f) + (q&0x7f)) > 0x7f);
	cpu.CC_H = ((p&0xf) + (q&0xf) > 0xf);
	return (result & 0xff);
    }

    final public int inc8(int p)
    {
	int result = ((p&0xff) + 1)&0xffff;
	setNZ8(result);
	cpu.CC_V = (result > 0xff) != (((p&0x7f) + 1) > 0x7f);
	return (result & 0xff);
    }
 
    /**
     * Decrement a 16-bit value setting ONLY the Z flag.
     */
    final public int dec16(int p)
    {
	int result = p - 1;
	cpu.CC_Z = (result == 0);
	return (result & 0xffff);
    }

    final public int eor8(int p, int q)
    {
	int result = ((p&0xff) ^ (q&0xff))&0xffff;
	setNZ8(result);
	cpu.CC_C = false;
	return (result & 0xff);
    }

    final public int sub8(int p, int q)
    {
	p = p&0xff;
	q = q&0xff;
	int result = (p - q)&0xffff;
	setNZ8(result);
	cpu.CC_C = (q > p);
	cpu.CC_V = (result > 0xff) != ((p&0x7f) < (q&0x7f));
	return (result & 0xff);
    }

    final public int sub16(int p, int q)
    {
	int result = ((p&0xffff) - (q&0xffff))&0xfffff;
	setNZ16(result);
	cpu.CC_C = (result > 0xffff);
	cpu.CC_V = (result > 0xffff) != ((p&0x7fff) < (1+(q&0x7fff)));
	return (result & 0xffff);
    }

    final public int com8(int p)
    {
	int result = (~p)&0xff;
	setNZ8(result);
	cpu.CC_C = true;
	cpu.CC_V = false;
	return result;
    }

    final void setNZ8(int d)
    {
	cpu.CC_Z = (d&0xff)==0;
	cpu.CC_N = (d&0x80)!=0;
    }

    final void setNZ16(int d)
    {
	cpu.CC_Z = (d)==0;
	cpu.CC_N = (d&0x8000)!=0;
    }

    final public int and8(int p, int q)
    {
	int result = (p&q&0xff);
	setNZ8(result);
	cpu.CC_V = false;
	return result;
    }

    final public int or8(int p, int q)
    {
	int result = (p|q)&0xff;
	setNZ8(result);
	cpu.CC_V = false;
	return result;
    }
    final public int add16(int p, int q)
    {
	int result = ((p&0xffff) + (q&0xffff))&0xffff;
	cpu.CC_Z = ((result&0xffff) == 0);
	cpu.CC_N = ((result&0x8000) == 0x8000);
	cpu.CC_C = (result > 0xffff);
	cpu.CC_V = (result > 0xffff) != (((p&0x7fff) + (q&0x7fff)) > 0x7fff);
	return (result & 0xffff);
    }

    final int getImmediateOperand()
    {
	return  (int) (mem.getCell(cpu.PC++).read());
    }

    final int getMask() //Just like an IMM8 operand...
    {
	return  getImmediateOperand();
    }

    final int getIMM8Operand()
    {
	return  (int) (mem.getCell(cpu.PC++).read());
    }

    final int getIMM16Operand()
    {
	int hi, lo;
	hi = (int)(mem.getCell(cpu.PC++).read());
	lo = (int)(mem.getCell(cpu.PC++).read());
	return ((hi&0xff)<<8) + (lo&0xff);
    }

    final int getIX8Operand()
    {
	int offset;
	int address;
	offset = (int)(mem.getCell(cpu.PC++).read())&0xff;
	address = cpu.IX + offset;
	cpu.incrementCycles();
	return (int) (mem.getCell(address).read());
    }

    final int getIX16Operand()
    {
	int offset;
	int address;
	int hi, lo;
	offset = (int)(mem.getCell(cpu.PC++).read())&0xff;
	address = cpu.IX + offset;
	cpu.incrementCycles();
	hi = (int)(mem.getCell(address++).read());
	lo = (int)(mem.getCell(address).read());
	return ((hi&0xff)<<8) + (lo&0xff);
    }


    final int getIX2Operand()
    {
	return getIX8Operand();
    }

    final int getIX3Operand()
    {
	return getIX8Operand();
    }

    final int getIY2Operand()
    {
	return getIY8Operand();
    }

    final int getIY3Operand()
    {
	return getIY8Operand();
    }

    final int getIY8Operand()
    {
	int offset;
	int address;
	offset = (int)(mem.getCell(cpu.PC++).read())&0xff;
	address = cpu.IY + offset;
	cpu.incrementCycles();
	return (int) (mem.getCell(address).read());
    }

    final int getIY16Operand()
    {
	int offset;
	int address;
	int hi, lo;
	offset = (int)(mem.getCell(cpu.PC++).read())&0xff;
	address = cpu.IY + offset;
	cpu.incrementCycles();
	hi = (int)(mem.getCell(address++).read());
	address &= 0xffff;
	lo = (int)(mem.getCell(address).read());
	return ((hi&0xff)<<8) + (lo&0xff);
    }

    final int getEXT8Operand()
    {
	int addressL;
	int addressH;
	int address;
	int op;
	//	addressH = (int) (cpu.mem.getCell(cpu.PC++).read());
	addressH = (int) (mem.getCell(cpu.PC++).read());
	addressH = addressH&0xff;
	//	addressL = (int) (cpu.mem.getCell(cpu.PC++).read());
	addressL = (int) (mem.getCell(cpu.PC++).read());
	addressL = addressL&0xff;
	address = (0xffff)&((addressH<<8)+addressL);
	//	op = (int)  (cpu.mem.getCell(address).read());
	op = (int)  (mem.getCell(address).read());
	return op;
    }

    final int getEXT16Operand()
    {
	int addressL;
	int addressH;
	int address;
	int hi, lo;
	addressH = (int) (mem.getCell(cpu.PC++).read());
	addressH = addressH&0xff;
	addressL = (int) (mem.getCell(cpu.PC++).read());
	addressL = addressL&0xff;
	address = (0xffff)&((addressH<<8)+addressL);
	hi = (int)  (mem.getCell(address++).read());
	lo = (int)(mem.getCell(address).read());
	return ((hi&0xff)<<8) + (lo&0xff);
    }

    final int getDIR8Operand()
    {
	int address;
	int op;
	address = (int) (mem.getCell(cpu.PC++).read());
	op = (int)  (mem.getCell(address).read());
	return op;
    }
    
    final int getDIR3Operand()
    {
	Utility.fatal("getDIR3Operand not done");
	return -2;
    }
    
    
    final int getDIR16Operand()
    {
	int address;
	int hi, lo;
	address = (int) (mem.getCell(cpu.PC++).read());
	hi = (int)  (mem.getCell(address++).read());
	lo = (int)(mem.getCell(address).read());
	return ((hi&0xff)<<8) + (lo&0xff);
    }

    public int getOperand()
    {
	return 0;
    }

    final public String toDisassembledString()
    {
	return getOpSymbolic() + toOperandString();
    }

    protected String getOpSymbolic()
    {
	return "Sub class must implement this!";
    }

    final protected String toOperandString()
    {
	int location = cpu.PC;
	int operand;
	int addressLo;
	int addressHi;
	int address;
	int mask;
	int offset;
	int branchOffset;
	int hi, lo;
	int mode = getAddressingMode();
	String s;
	switch (mode) {
	case Instruction.IMM8:
	    operand =  mem.getCell(location).get();
	    //s = Integer.toHexString((operand&0xff) + 0x100).substring(1);
	    s = Utility.int8ToHex(operand);
	    s = " #" + s;
	    break;
	case Instruction.INH:
	    s = "";
	    break;
	case Instruction.REL:
	    offset =  mem.getCell(location).get();
	    offset = (byte) (offset&0xff);
	    //s=Integer.toHexString(offset + location + 0x10001).substring(1);
	    s = Utility.int16ToHex(offset + location);
	    s = " " + s;
	    break;
	case Instruction.DIR8:
	case Instruction.DIR16:
	    operand = mem.getCell(location).get();
// 	    operand = (byte) (operand&0xff);
// 	    s = Integer.toHexString(operand + 0x10000).substring(1);
	    s = Utility.int8ToHex(operand&0xff);
	    s = " " + s;
	    break;
	case Instruction.EXT8:
	case Instruction.EXT16:
	    addressHi = mem.getCell(location).get();
	    addressLo = (mem.getCell(location+1)).get();
	    operand =  ((addressHi<<8)+addressLo)&0xffff;
	    //s = Integer.toHexString(operand + 0x10000).substring(1);
	    s = Utility.int16ToHex(operand);
	    s = " " + s;
	    break;
	case Instruction.IX8:
	case Instruction.IX16:
	    offset = mem.getCell(location).get();
	    s = " " + Integer.toHexString(offset+0x100).substring(1) + ",x";
	    break;
	case Instruction.IY8:
	case Instruction.IY16:
	    offset = mem.getCell(location).get();
	    //s = " " + Integer.toHexString(offset+0x100).substring(1) + ",y";
	    s = " " + Utility.int8ToHex(offset) + ",y";
	    break;
	case Instruction.IMM16:
	    hi =  mem.getCell(location++).get();
	    lo =  mem.getCell(location).get();
	    operand =  ((hi&0xff)<<8) + (lo&0xff);
	    //s = Integer.toHexString((operand&0xffff) + 0x10000).substring(1);
	    s = Utility.int16ToHex(operand);
	    s = " #" + s;
	    break;
	case Instruction.IX2:
	    offset =  mem.getCell(location++).get();
	    mask =  mem.getCell(location++).get();
	    s = " ";
	    //s += Integer.toHexString((offset&0xff) + 0x100).substring(1);
	    s += Utility.int8ToHex(offset);
	    s = s + ",X ";
	    s += Integer.toHexString((mask&0xff) + 0x100).substring(1);
	    break;
	case Instruction.IY2:
	    offset =  mem.getCell(location++).get();
	    mask =  mem.getCell(location++).get();
	    s = " ";
	    s += Integer.toHexString((offset&0xff) + 0x100).substring(1);
	    s = s + ",Y ";
	    s += Integer.toHexString((mask&0xff) + 0x100).substring(1);
	    break;
	case Instruction.DIR2:
	    address =  mem.getCell(location++).get();
	    mask =  mem.getCell(location++).get();
	    s = " ";
	    s += Integer.toHexString((address&0xff) + 0x100).substring(1);
	    s = s + " ";
	    s += Integer.toHexString((mask&0xff) + 0x100).substring(1);
	    break;
	case Instruction.DIR3:
	    address =  mem.getCell(location++).get();
	    mask =  mem.getCell(location++).get();
	    offset =  mem.getCell(location).get();
	    offset = (byte) (offset&0xff);
	    s = " ";
	    s += Utility.int8ToHex(address);
	    s += " ";
	    s += Utility.int8ToHex(mask);
	    s += " ";
	    s += Utility.int16ToHex(location + 1 + offset);
	    break;
	case Instruction.IX3:
	    offset =  mem.getCell(location++).get();
	    mask =  mem.getCell(location++).get();
	    branchOffset =  mem.getCell(location++).get();
	    branchOffset = (byte) (branchOffset&0xff);
	    s = " ";
	    s += Utility.int8ToHex(offset);
	    s += ",X ";
	    s += Utility.int8ToHex(mask);
	    s += " ";
	    s += Utility.int16ToHex(location + branchOffset);
	    break;
	case Instruction.IY3:
	    offset =  mem.getCell(location++).get();
	    mask =  mem.getCell(location++).get();
	    branchOffset =  mem.getCell(location++).get();
	    branchOffset = (byte) (branchOffset&0xff);
	    s = " ";
	    s += Utility.int8ToHex(offset);
	    s += ",Y ";
	    s += Utility.int8ToHex(mask);
	    s += " ";
	    s += Utility.int16ToHex(location + branchOffset);
	    break;
	default:
	    System.out.println("OOPS!!!---unknown addressing Mode: " + mode);
	    s = "";
	}
	return s;
    }

    int getAddressingMode()
    {
	return 0;
    }

    int getOperandAddress()
    {
	Utility.fatal("Getting operand address");
	return 0;
    }

    int asl8(int op)
    {
	op = op<<1;
	cpu.CC_C = op > 0xff;
	return (op&0xff);
    }

    int rol8(int op)
    {
	
	op = (op<<1) + (cpu.CC_C ? 1 : 0);
	cpu.CC_C = op > 0xff;
	op &= 0xff;
	cpu.CC_Z = op == 0;
	return (op);
    }
    int ror8(int op)
    {
	op = op + (cpu.CC_C ? 0x100 : 0);
	cpu.CC_C = (op&1) == 1;
	op = op>>1;
	op &= 0xff;
	cpu.CC_Z = op == 0;
        cpu.CC_N = ((op&0x80) == 0x80);
	return (op);
    }

    int asr8(int op)
    {
	cpu.CC_C = (op&1) == 1;
	op = op>>1;
	return (op&0xff);
    }
    int lsr8(int op)
    {
	
	cpu.CC_C = (op&1) != 0;
	op = op>>1;
	op &= 0x7f;
	setNZ8(op);
	//NEED TO SET V bit!!!
	return op;
    }

    void writeOperand8(int op, int address)
    {
	mem.getCell(address).write(op);
    }

    void writeOperand16(int op, int address)
    {
	writeOperand8((op&0xff00)>>8, address);
	writeOperand8((op&0xff), (address+1)&0xffff);
    }

    int read8(int address)
    {
	int data;
	data = (int) mem.getCell(address).read();
	data &= 0xff;
	return data;
    }

    int read16(int address)
    {
	int hi, lo;
	hi = read8(address);
	lo = read8((address+1)&0xffff);
	return (hi<<8)+lo;
    }

    final int getOperandAddressExt()
    {
	int addressL;
	int addressH;
	int address;
	addressH = (int) (mem.getCell(cpu.PC++).read());
	addressL = (int) (mem.getCell(cpu.PC++).read());
	address = (0xffff)&((addressH<<8)+addressL);
	return address;
    }

    final int getOperandAddressDir()
    {
	int address;
	address = (int) (mem.getCell(cpu.PC++).read());
	return address;
    }

    final int getOperandAddressIX()
    {
	int offset;
	int address;
	int hi, lo;
	offset = (int)(mem.getCell(cpu.PC++).read())&0xff;
	address = cpu.IX + offset;
	cpu.incrementCycles();
	return address;
    }

    final int getOperandAddressIY()
    {
	int offset;
	int address;
	int hi, lo;
	offset = (int)(mem.getCell(cpu.PC++).read())&0xff;
	address = cpu.IY + offset;
	cpu.incrementCycles();
	return address;
    }

    int getByteAt(int address)
    {
	int data;
	data = (int)(mem.getCell(address).read())&0xff;
	return data;
    }
}

