package com.kclowes.sim11;
import java.util.*;
/** Simulates the 6811 CPU.
 *
 * @version 1.4 (October 30, 2002)
 * @author Ken Clowes
 */

public class CPU6811 {
    private static final CPU6811 instance = new CPU6811();

    private static final Instruction stopOnSWIInstruction = new StopOnSwiInh();
//     private static final boolean DEBUG = true;
    private static final boolean DEBUG = false;
    //Statistics maintained by the class
    private int totalWrites = 0;
    private int totalReads = 0;
    private int totalNInstructions = 0;
    private long nCycles = 0;
    private DeltaList pendingTimedEvents = new DeltaList();
    private boolean stopOnSWI = false;

    //Register values
    //!!!KLUDGE!!! These SHOULD be declared private (non-static)
    // i.e. private int AccA = 0x12;
    static int AccA = 0x12;
    static int AccB = 0x34;
    public static int PC = 0x6000;  //Standard starting address at RyersonU
    public static int IX = 0;
    public static int IY = 0;
    private  int SP = 0x41;   //?Standard BUFFALO monitor stack?
    public static boolean CC_Z = false; //
    public static boolean CC_N = false;
    public static boolean CC_C = false;
    public static boolean CC_V = false;
    public static boolean CC_H = false;
    public static boolean CC_I = true;
    static boolean CC_X = true;

    static final MemorySpace mem = MemorySpace64K.getInstance();
    private PendingInterrupts pendingInterrupts =
	PendingInterrupts.getInstance();
    public final static int RUNNING = 1;
    public static final int IDLE = 2;
    private int state = IDLE;

    private boolean[] brkPoints = new boolean[0x10000];
    
    //Singleton, so private constructor only.
    private CPU6811()
    {
    }

    // Methods to get/set registers
    /** Self-explanatory */
    public int getAccD()
    {
	return  (((AccA<<8)&0xff00) + AccB);
    }

    /** Set Accumulator D value giving individual Accumulator A and B values.
     *
     * @param a Value of Accumulator A
     * @param b Value of Accumulator B
     */ 
    public static void setAccD(int a, int b)
    {
	AccA = a;
	AccB = b;
    }

    /** Set Accumulator D value.
     *
     * @param d Value of Accumulator D
     */ 
    public void setAccD(int d)
    {
	AccA = d>>8;
	AccB = d&0xff;
    }

    final public int getPC()
    {
	return PC;
    }

    final public void setPC(int pc)
    {
	PC = pc;
    }

    /** Self-explanatory */
    final public int getAccA()
    {
	return AccA;
    }

    /** Self-explanatory */
    final public void setAccA(int i)
    {
	AccA = i;
    }

    /** Self-explanatory */
    final public int getAccB()
    {
	return AccB;
    }

    /** Self-explanatory */
    final public void setAccB(int i)
    {
	AccB = i;
    }

    /** Self-explanatory */
    final public int getIX()
    {
	return IX;
    }

    /** Self-explanatory */
    final public void setIX(int i)
    {
	IX = i;
    }

    /** Self-explanatory */
    final public int getIY()
    {
	return IY;
    }

    /** Self-explanatory */
    final public void setIY(int i)
    {
	IY = i;
    }

    /** Self-explanatory */
    final public int getSP()
    {
	return SP;
    }

    /** Self-explanatory */
    final public void setSP(int i)
    {
	SP = i;
    }

    /** Self-explanatory */
    final public void incrementSP()
    {
	int oldSP = SP;
	int newSP = ++SP;
	SP = newSP;
	if ((SP&0xffff0000) != 0) {
	    SP = SP&0xffff;
	    Warn.StackUnderOverFlow(oldSP, newSP);
	}
	SP &= 0x0ffff;
    }

    /** Self-explanatory */
    final public void incrementSPby2()
    {
	int oldSP = SP;
	int newSP = SP + 2;
	SP = newSP;
	if ((SP&0xffff0000) != 0) {
	    SP = SP&0xffff;
	    Warn.StackUnderOverFlow(oldSP, newSP);
	}
	SP &= 0x0ffff;
    }

    /** Self-explanatory */
    final public void decrementSP()
    {
	int oldSP = SP;
	int newSP = SP - 1;
	SP = newSP;
	if ((SP&0xffff0000) != 0) {
	    SP = SP&0xffff;
	    Warn.StackUnderOverFlow(oldSP, newSP);
	}
	SP &= 0x0ffff;
    }

    /** Self-explanatory */
    final public void decrementSPby2()
    {
	int oldSP = SP;
	int newSP = SP - 2;
	SP = newSP;
	if ((SP&0xffff0000) != 0) {
	    SP = SP&0xffff;
	    Warn.StackUnderOverFlow(oldSP, newSP);
	}
	SP &= 0x0ffff;
    }

    public final void setCC(int ccr)
    {
	CC_N = (ccr&0x08) != 0;
	CC_Z = (ccr&0x04) != 0;
	CC_V = (ccr&0x02) != 0;
	CC_C = (ccr&0x01) != 0;
	CC_I = (ccr&0x10) != 0;
	CC_H = (ccr&0x20) != 0;
	CC_X = (ccr&0x40) != 0;
    }

    /** Sets the CC I (Interrupt Inhibit) bit.  If set to <code>true</code>
     * (1), maskable interrupts are not recognized; otherwise, they are.
     */
    public final void setCC_I(boolean v)
    {
	CC_I = v;
    }
    

    /** Gets the CC I (Interrupt Inhibit) bit.  If set to <code>true</code>
     * (1), maskable interrupts are not recognized; otherwise, they are.
     */
    public final boolean getCC_I()
    {
	return CC_I;
    }

    /** Sets the CC X (XInterrupt Inhibit) bit.  If set to <code>true</code>
     * (1), maskable interrupts are not recognized; otherwise, they are.
     */
    public final void setCC_X(boolean v)
    {
	CC_X = v;
    }
    

    /** Gets the CC DXI (Nonmaskable Interrupt Inhibit) bit.  If set to <code>true</code>
     * (1), maskable interrupts are not recognized; otherwise, they are.
     */
    public final boolean getCC_X()
    {
	return CC_X;
    }

    /** Sets the CC N (Negative) bit.
     */
    final public void setCC_N(boolean v)
    {
	CC_N = v;
    }
    

    /** Gets the CC N (Negative) bit.
     */
    final public boolean getCC_N()
    {
	return CC_N;
    }

    /** Sets the CC Z (Zero) bit.
     */
    final public void setCC_Z(boolean v)
    {
	CC_Z = v;
    }
    

    /** Gets the CC Z (Zero) bit.
     */
    public boolean getCC_Z()
    {
	return CC_Z;
    }

    /** Sets the CC V (Overflow) bit.
     */
    final public void setCC_V(boolean v)
    {
	CC_V = v;
    }
    

    /** Gets the CC V (Overflow) bit.
     */
    final public boolean getCC_V()
    {
	return CC_V;
    }
    /** Sets the CC V (Carry) bit.
     */
    final public void setCC_C(boolean v)
    {
	CC_C = v;
    }
    

    /** Gets the CC C (Carry) bit.
     */
    final public boolean getCC_C()
    {
	return CC_C;
    }
    
    public static int getCC()
    {
	int ccr = 0;
	ccr += CC_N ? 0x08 : 0;
	ccr += CC_Z ? 0x04 : 0;
	ccr += CC_V ? 0x02 : 0;
	ccr += CC_C ? 0x01 : 0;
	ccr += CC_I ? 0x10 : 0;
	ccr += CC_H ? 0x20 : 0;
	ccr += CC_X ? 0x40 : 0;
	return ccr;
    }


    //    public static final int STOPPED_BY_USER = 1;
    public static final int MAX_INSTRUCTIONS_REACHED = 2;

    private int simDoneCode = 0;
    private int maxInst = 20;
    private boolean stopSim = false;

    public static CPU6811 getInstance() {
	return instance;
    }

    public final void incrementCycles()
    {
	nCycles++;
	TimedEvent event;
	event = (TimedEvent) pendingTimedEvents.tickAndRemove();
	if (event != null) {
	    event.doAction();
	    while(pendingTimedEvents.isNextNow()) {
		event = (TimedEvent) pendingTimedEvents.tickAndRemove();
		event.doAction();
	    }
	}
    }

    public final void incrementReadCycles()
    {
	totalReads++;
	incrementCycles();
    }

    public final void incrementWriteCycles()
    {
	totalWrites++;
	incrementCycles();
    }

    /** Self explanatory */
    public final long getNCycles()
    {
	return nCycles;
    }

    /** Self explanatory */
    public final int getNReads()
    {
	return totalReads;
    }

    /** Self explanatory */
    public final int getNWrites()
    {
	return totalWrites;
    }

    /** Self explanatory */
    public final int getNInstructions()
    {
	return totalNInstructions;
    }


    /** Executes a single instruction.  The side effects (other than
     * changes in registers) are returned.
     * @return InstructionEffects containing disassembled instruction
     *         and memory location modified.
     */

    public final InstructionEffects step()
    {
	Instruction inst;
	InstructionEffects effects = new InstructionEffects();
	inst = fetchInstruction();
	effects.disassembled = inst.toDisassembledString();
	effects.memModified = inst.execute();
	totalNInstructions++;
	checkInterrupts();
	return effects;
    }

    public final int fastStep()
    {
	int rc;
	Instruction inst;
	inst = fetchInstruction();
	totalNInstructions++;
	rc = inst.execute();
	checkInterrupts();
	return rc;
    }

    /** Runs the simulator at full speed.  The simulation ends when:
     *  <ul>
     *    <li>
     *	    a STOP instruction is executed;
     *    </li>
     *    <li>
     *      the maximum number of instuctions is met;
     *    </li>
     *    <li>
     *      the thread is interrupted by invoking <code>stopSim()</code>;
     *    </li>
     *    <li>
     *      a breakpoint is met.
     *    </li>
     *  </ul>
     *
     * @param maxNInstructions The max number of instructions to execute.
     */

    public Runnable go(int maxNInstructions)
    {
	setState(RUNNING);
	stopSim = false; //Is this needed???
	maxInst = maxNInstructions - 1;

	Thread goThread = new Thread ( new Runnable() {
		public void run() {doFastSimulation(); }
	} );
	goThread.setPriority(Thread.MIN_PRIORITY);

	/* Single step one instruction (it may be a breakpoint)
	 * before starting the thread.
	 */
	step();

	//	goThread.start();
	return goThread;
    }

    static private Instruction pushAll = new DummyPushAll();
    static private DummyRead16 read16 = new DummyRead16();
    static int XIRQ_VECTOR = 0xFFF4;

    private void checkInterrupts()
    {
	if (!CC_I) {
	    if (pendingInterrupts.size() > 0) {
		Iterator iter = pendingInterrupts.iterator();
		while(iter.hasNext()) {
		    InterruptRequest pending = (InterruptRequest) iter.next();
		    if (pending.isActive()) {
			if (DEBUG) {
			    System.out.println("Handling active interrupt: " +
					       pending + "#pending: " +
					       pendingInterrupts.size());
			}
			pending.handle();
		    } else {
			iter.remove();
		    }
		}
	    }
	}
	if (!CC_X) {
	    Cell porta = mem.getCell(0x1000);
	    System.out.println("X interrupts enabled, porta:" +
			       Utility.int8ToHex(porta.get()));
	    if (((porta.get())&0x08) == 0) { //Active XIRQ request
		if (DEBUG) {
		    System.out.println("Handling XIRQ");
		}
		pushAll.execute();
		CC_X = true;
		read16.setAddress(XIRQ_VECTOR);
		int newPC = read16.execute();
		setPC(newPC);
	    }
	}
    }
    
    /** Stops full speed simulation.  The <code>StopReason</code>
     * is set to <code>StopReason.STOPPED_BY_USER</code>.
     */
    public synchronized void stopSim()
    {
	stopSim = true;
	setStopReason(StopReason.STOPPED_BY_USER);
    }

    
    /** Stops full speed simulation.
     * @param reason
     */
    public synchronized void stopSim(StopReason reason)
    {
	stopSim = true;
	setStopReason(reason);
    }

    final static boolean DEBUG_FASTSIM = false;

    /** Wait until the CPU is not in the Running state.
     */
    public synchronized void waitNotRunning()
    {
	while(getState() == CPU6811.RUNNING) {
	    try {
		wait();
		//Thread.sleep(123);
	    } catch (InterruptedException e) {}
	}
    }

    private void doFastSimulation()
    {
	int stepRet = 0;
	if (maxInst == -1) {
	    while(!stopSim) {
		if (!brkPoints[PC]) {
		    stepRet = fastStep();
		} else { //breakpoint hit
		    setStopReason(StopReason.STOPPED_BY_BRK);
		    setState(IDLE);
		    return;
		}
	    }
	} else {
	    for(int i = 0; (i < maxInst) && !stopSim; i++) {
		if (!brkPoints[PC]) {
		    stepRet = fastStep();
		} else { //breakpoint hit
		    setStopReason(StopReason.STOPPED_BY_BRK);
		    setState(IDLE);
		    return;
		}
	    }
	}
	if (stopSim) {
	    if (stepRet == Instruction.STOP) {
		setStopReason(StopReason.STOPPED_BY_STOP);
	    } else if (stepRet == Instruction.STOP_ON_SWI) {
		setStopReason(StopReason.STOPPED_BY_SWI);
	    } else if (stopReason == StopReason.STOPPED_BY_WARNING) {
		;
	    } else {
		setStopReason(StopReason.STOPPED_BY_USER);
	    }
	} else {
	    simDoneCode = MAX_INSTRUCTIONS_REACHED;
	    setStopReason(StopReason.STOP_NUM_INSTRUCTIONS);
	}
	setState(IDLE);
    }
    
    public int getStopCode()
    {
	return simDoneCode;
    }

    private final int fetchOpCode()
    {
	return mem.getCell(CPU6811.PC++).read();
    }

    private final Instruction fetchInstruction()
    {
	int opCode;
	Instruction inst;
	opCode = fetchOpCode();
	switch (opCode) {
	case 0x18:
	    opCode = fetchOpCode();
	    inst = OpCodes18.Opcodes18[opCode];
	    break;
	case 0xcd:
	    opCode = fetchOpCode();
	    inst = OpCodescd.Opcodescd[opCode];
	    break;
	case 0x1a:
	    opCode = fetchOpCode();
	    inst = OpCodes1a.Opcodes1a[opCode];
	    break;
	case 0x3f: // SWI instruction handled specially (Oct 8, 2003)
	    if (stopOnSWI) {
		inst = stopOnSWIInstruction;
	    } else {
		inst = OpCodes.Opcodes[opCode];
	    }
	    break;
	default:
	    inst = OpCodes.Opcodes[opCode];
	}
	return inst;
    }

    public final int getTotalReads()
    {
	return totalReads;
    }

    public final int getTotalWrites()
    {
	return totalWrites;
    }

    public final synchronized int getState()
    {
	return state;
    }

    public final synchronized void setState(int s)
    {
	state = s;
	notifyAll();
    }

    private StopReason stopReason = null;

    /** Set stop reason */
    private void setStopReason(StopReason reason)
    {
	stopReason = reason;
    }

    /** Get the reason why the last full-speed simulation terminated.
     */
    public final StopReason getStopReason()
    {
	return stopReason;
    }

    /** Determine if an address is a breakpoint.
     *
     * @param address Address to test.
     * @return true or false.
     */
    public boolean isBreakpoint(int address)
    {
	return brkPoints[address];
    }

    /** Change the state of a breakpoint. Returns the new state.
     *
     * @param address Address to toggle Breakpoint state.
     * @return true or false.
     */
    public boolean toggleBreakpoint(int address)
    {
	brkPoints[address] = !brkPoints[address];
	return brkPoints[address];
    }

    /**Reason that full speed simulation was terminated.
     * This class cannot be instantiated.
     */
    public static class StopReason {
	private String reason;
	public static final StopReason STOP_NUM_INSTRUCTIONS =
	    new StopReason("Number instructions met");
	public static final StopReason STOPPED_BY_USER =
	    new StopReason("Stopped by user");
	public static final StopReason STOPPED_BY_STOP =
	    new StopReason("Stopped by \"STOP\" instruction");
	public static final StopReason STOPPED_BY_BRK =
	    new StopReason("Stopped by breakpoint");
	public static final StopReason STOPPED_BY_SWI =
	    new StopReason("Stopped by SWI instruction");
	public static final StopReason STOPPED_BY_WARNING =
	    new StopReason("Stopped by Warning");

	private StopReason(String reason)
	{
	    this.reason = reason;
	}

	public String toString()
	{
	    return reason;
	}
    }

    /** Add a timed event to pending events
     *
     * @see TimedEvent TimedEvent
     */
    public void addTimedEvent(TimedEvent timedEvent)
    {
	pendingTimedEvents.add(timedEvent);
    }

    /** Remove all pending Timed Events.  Used only for testing.
     */
    public void removeALLTimedEvents()
    {
	pendingTimedEvents = new DeltaList();
    }

    /** Get the underlying pending timed event list.
     * Used only for testing...should be removed or replaced...
     */
    public DeltaList getDeltaList()
    {
	return pendingTimedEvents;
    }


    public void reset()
    {
	//Get reset vector
	int vHi = mem.getCell(0xFFFE).get();
	int vLo = mem.getCell(0xFFFF).get();
	int vector = vHi<<8 + vLo;
	PC = vector;
	for(int i = 0; i < 0x10000; i++) {
	    Cell cell;
	    cell = mem.getCell(i);
	    if (cell instanceof SpecialByteCell) {
		cell.reset();
	    }
	}
	totalWrites = 0;
	totalReads = 0;
	totalNInstructions = 0;
	nCycles = 0;
	pendingTimedEvents = new DeltaList();
	    
    }

    /** Configure for stop on SWI instruction.
     * @param b true: stop on SWI
     */
    public void setStopOnSWI(boolean b)
    {
	stopOnSWI = b;
    }

    /** Determine if stop on SWI instruction.
     * @return true or false.
     */
    public boolean isStopOnSWI()
    {
	return stopOnSWI;
    }
}
