package com.kclowes.sim11;
import java.util.*;
/** Collection of pending maskable interrupts.
 *
 * @see InterruptRequest
 * @version 1.1 (November 23, 2002) (PREV:August 20, 2002)
 * @author Ken Clowes
 */
public class PendingInterrupts {
    /* Uncomment ONE of the following definitions. */
    private static boolean DEBUG = false;
    //private static boolean DEBUG = true;

    private static final PendingInterrupts INSTANCE = new PendingInterrupts();
    private Collection pending = new ArrayList();
    
    //Singleton, so private constructor only.
    private PendingInterrupts()
    {
    }

    /** Get the singleton instance of the PendingInterrupts class.
     */
    public static final PendingInterrupts getInstance()
    {
	return INSTANCE;
    }

    /** Return number of pending interrupts.
     */
    int size()
    {
	return pending.size();
    }

    /** Add an interrupt request.  The request is added unless a request
     * with the same vector is already in the pending list.
     *
     * @param request The interrupt request.
     */
    public final void add(InterruptRequest request)
    {
	if (DEBUG) {
	    System.out.println("Adding (maybe) interrupt request: " +
			       request);
	    System.out.println("Vector: " +
			       Utility.int16ToHex(request.getVector()));
	}
	int vectorToAdd = request.getVector();

        //Determine if a request with the same vector exists.
	boolean alreadyExists = false;
	for(Iterator iter = pending.iterator(); iter.hasNext();) {
	    if (((InterruptRequest)iter.next()).getVector() == vectorToAdd) {
		alreadyExists = true;
		break;
	    }
	}

        //Add new request ONLY IF another one is already on the pending list.
	if (!alreadyExists) {
	    pending.add(request);
	}
    }

    /** Get iterator for pending interrupt requests.
     */
    public final Iterator iterator()
    {
	return pending.iterator();
    }
}

