package com.kclowes.sim11;
/**
 * This class implements a byte of ROM.
 *
 * @version 1.1 (May 20, 2002)
 * @author Ken Clowes
*/
public class ROMByteCell extends AbstractByteCell
{
    private final static int DEFAULT_CONTENTS = 0xff;

    /** Creates a byte of ROM with known address and initial contents.
     * @param address The address where the byte is mapped.
     * @param contents The contents of the ROM byte.
     */

    public ROMByteCell(int address, int contents)
    {
	super(address, contents);
	this.address = address;
	this.contents = contents;
	nWrites = 0;
	nReads = 0;
    }


    /** Creates a byte of ROM with known address. The contents are itialized
     * to 0xff.
     * @param address The address where the byte is mapped.
     */
    public ROMByteCell(int address)
    {
	this(address, 0xff);
    }

    /** Returns the contents of the byte.  The number of times the
     * cell is read is tracked.
     */
    public int read()
    {
	nReads++;
	cpu.incrementCycles();
	return  contents;
    }

    /** Simulates a write bus cycle to the byte. The ROM byte does
     * not change its contents. The number of times
     * the byte is written to is tracked.
     *
     * @param contents What will NOT be written to the ROM byte.
     */

    public void write(int contents)
    {
	Warn.ROMWrite(this);
	nWrites++;
	cpu.incrementWriteCycles();
    }

    /** Simulates a write bus cycle to the byte. The ROM byte does
     * not change its contents. The number of times
     * the byte is written to is tracked.
     *
     * @param contents What will NOT be written to the ROM byte.
     */
    public void write(byte contents)
    {
	write((int) contents);
    }

}
