package com.kclowes.sim11;
/** Implements an event to fire when it is removed from a DeltaList.
 *
 *
 * @version 1.0 (July 31, 2002)
 * @author Ken Clowes
 */

public class TimedEvent implements DeltaList.DeltaTimed {
    private int delta;
    private TimedAction action = null;

    public TimedEvent(int delta, TimedAction action)
    {
	assert delta >= 0;
	this.delta = delta;
	this.action = action;
    }
    
    TimedEvent(int delta)
    {
	this(delta, null);
    }

    /**
       Get the delta time for this event.  Note that the delta time is
       <em>not</em> is an absolute time; it is a <emph>relative</emph>
       time.

       <p>
       If the event is the next one to be fired, its absolute time is
       its delta time PLUS the absolute current time.  If it is not
       the next event, its absolute time can be determined by adding
       its delta time with the absolute time of its preceeding event.
       </p>
       @return relative delta time
     */
    public int getDelta()
    {
	return delta;
    }

    public void setDelta(int delta)
    {
	assert delta >= 0;
	this.delta = delta;
    }

    public String toString()
    {
	String r;
	r =  "Delta: " + getDelta() + " Action: " + action;
	return r;
    }

    /** Action to perform when the timed event becomes active.
     */
    public void doAction()
    {
	action.doAction();
    }
}
