package com.kclowes.sim11;
/**
 * This class implements the contents of an address in MemorySpace
 * that has not been mapped to any real byte.
 *
 * @version 1.1 (May 20, 2002)
 * @author Ken Clowes
*/

public class UnusedByteCell extends AbstractByteCell
{
    private final static int DEFAULT_CONTENTS = 0x01;

    /** Creates a cell with known address.
     * @param address The address where the byte is mapped.
     */


    public UnusedByteCell(int address)
    {
	super(address);
	this.address = address;
	contents = DEFAULT_CONTENTS;
	nWrites = 0;
	nReads = 0;
    }

    /** Returns the contents of the byte.  The number of times the
     * cell is read is tracked.
     */
    public int read()
    {
	Warn.UnusedRead(this);
	nReads++;
	return  contents;
    }

    /** Simulates a write bus cycle to the unmapped location.
     * The number of times the byte is written to is tracked.
     * The contents of the unmapped location will not be changed (obviously!)
     *
     * @param contents <b>Ignored</b>
     */

    public void write(int contents)
    {
	Warn.UnusedWrite(this);
	nWrites++;
	cpu.incrementWriteCycles();
    }

    /** Simulates a write bus cycle to the unmapped location.
     * The number of times the byte is written to is tracked.
     * The contents of the unmapped location will not be changed (obviously!)
     *
     * @param contents <b>Ignored</b>
     */
    public void write(byte contents)
    {
	write((int) contents);
    }

    /** Returns the contents of the unmapped byte.  There is no tracking of the
     * number of times the cell contents are obtained.
     */
    public int get()
    {
	return  contents;
    }

    /** Setting an unmapped location does nothing. */
    public void set(int contents)
    {
    }

}
