package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the 6811 ADC subsystem.
 *  It creates and maps the special memory cells (i.e. device registers):
 *  <ul>
 *    <li>The ADC Control/Status register(ADCTL)
 *        {@link ADCControl ADCControl} and
 *    <li>The 4 ADC result registers {@link ADCResult ADCResult}.</li>
 *  </ul>
 *
 *  @version 1.0 (September 19, 2002)
 *  @author Ken Clowes
 */

public class  ADC {
    private MemorySpace mem = MemorySpace64K.getInstance();
    private CPU6811 cpu = CPU6811.getInstance();
    private ADCControl adcControl = new ADCControl(DevRegisters.ADCTL_ADDRESS);
    static ADC INSTANCE = new ADC();
    private ADCResult resultRegisters[] = new ADCResult[4];
    public static ADC getInstance(){return INSTANCE;}
    private ADC()
    {
 	adcControl = new ADCControl(DevRegisters.ADCTL_ADDRESS);
 	mem.setCell(DevRegisters.ADCTL_ADDRESS, adcControl);
	for(int i = 1; i <=4; i++) {
	    ADCResult result = new ADCResult(DevRegisters.ADCTL_ADDRESS+i);
	    resultRegisters[i-1] = result;
	    mem.setCell(DevRegisters.ADCTL_ADDRESS+i, result);
	}
	//System.out.println("ADC created");
    }

    public ADCControl getAdcControl() {
	return adcControl;
    }

    ADCResult [] getResultRegisters()
    {
	return resultRegisters;
    }
}

