package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implementation of an ADC result register.
 * <p>
 * </p>
 *
 *  @version 1.0 (September 18, 2002)
 *  @author Ken Clowes
 */

public class ADCResult
    extends SpecialByteCell
    implements Cell {
    
    MemorySpace mem = MemorySpace64K.getInstance();

    public ADCResult(int address)
    {
	super(address);
	set(0);
    }
    
    /** Simulates a write bus cycle to an ADC result register.
     * The number of times the byte is written to is tracked.
     * Otherwise, the write is ignored. (Perhaps a Warning should
     * be issued?)
     *
     * @param junk What is written to the Result Register.
     */
    public void write(int junk)
    {
	nWrites++;
	cpu.incrementWriteCycles();
    }

    final public int read()
    {
	nReads++;
	cpu.incrementReadCycles();
	return  get();
    }
    
    /**
     * String representation of this ADC result register.
     * The format is: <code>ADC Result at 0x<4-hexdigits></code>.
     */
    public String toString()
    {
	return "ADC Result at 0x" + Utility.int16ToHex(address);
    }

    public String getName()
    {
	int n = address - DevRegisters.ADCTL_ADDRESS - 1;
	return "ADCData" + n;
    }
}

