package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the Timer Compare Force (CFORC) device register.
 * <p>
 * The bits in the CFORC register are:
 * <pre>
 *   +-----------------------------------------------+
 *   |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |  Bit numbers
 *   +-----------------------------------------------+
 *    FOC1  FOC2  FOC3  FOC4  FOC5    0     0     0     Mneumonics
 *      0    0     0     0      0     0     0     0     Reset values
 * </pre>
 *
 * <p>
 *
 * </p>
 *  @version 1.0 (October 29, 2002)
 *  @author Ken Clowes
 */

public class Cforc
    extends SpecialByteCell
    implements Cell {

    protected MemorySpace mem = MemorySpace64K.getInstance();
    private static int DEFAULT_CONTENTS = 0x00;
    private static Cforc INSTANCE = null;
    private Tctl1 tctl1 = Tctl1.getInstance();
    private PortA portA = PortA.getInstance();
    private int channel;

    public static Cforc getInstance()
    {
	if (INSTANCE == null) {
	    INSTANCE = new Cforc(DevRegisters.CFORC_ADDRESS);
	}
	return INSTANCE;
    }

    private Cforc(int address) //Singleton...private constructor
    {
	super(address);
	mem.setCell(address, this);
	reset();
    }


    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written to is tracked. The '1' bits in the contents
     * that are written invoke the corresponding Output Compare action.
     *
     * @param contents What is written to the byte.
     */
    public void write(int contents)
    {
	
	super.write(contents);
	int cforceBits = get();
	if ((cforceBits & 0x80) == 0x80) {
	    channel = 1;
	    doPortA();
	}
	if ((cforceBits & 0x40) == 0x40) {
	    channel = 2;
	    doPortA();
	}
	if ((cforceBits & 0x20) == 0x20) {
	    channel = 3;
	    doPortA();
	}
	if ((cforceBits & 0x10) == 0x10) {
	    channel = 4;
	    doPortA();
	}
	if ((cforceBits & 0x08) == 0x08) {
	    channel = 5;
	    doPortA();
	}
    }

    /** Update bit in PortA as specified in TCTL1.
     * KLUDGE: code dup'ed in Cforc.java...argh...
     */
    private void doPortA()
    {
	int action;
	action = tctl1.getPortAAction(channel);
	switch (action) {
	case Tctl1.DO_NOTHING:
	    break;
	case Tctl1.TOGGLE:
	    portA.toggleBits(TOCAction.PORTA_BITS[channel]);
	    break;
	case Tctl1.CLEAR:
	    portA.clearBits(TOCAction.PORTA_BITS[channel]);
	    break;
	case Tctl1.SET:
	    portA.setBits(TOCAction.PORTA_BITS[channel]);
	    break;
	}
    }

    
    public String getName()
    {
	return "CFORC";
    }

    public void reset()
    {
	set(DEFAULT_CONTENTS);
    }

}

