package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the Ryerson LCD display.
 *  It creates and maps the special memory cells (i.e. device registers):
 *  <ul>
 *    <li>LCD data register.</li>
 *    <li>LCD control/status register.</li>
 *  </ul>
 *
 *
 * <p>
 * The data register is used to write charaters to the display.
 * </p>
 *
 * @see LCDControl
 * @see LCDData
 *
 *  @version 1.0 (August 29, 2002)
 *  @author Ken Clowes
 */

public class LCD {

    public static int LCD_BASE_ADDRESS = 0x1400;
    private MemorySpace mem = MemorySpace64K.getInstance();
    private static LCD INSTANCE = new LCD(LCD_BASE_ADDRESS);
    private int baseAddress;

    public static LCD getInstance()
    {
	return INSTANCE;
    }

    private LCD(int base) //Singleton -- no public constructor
    {
	baseAddress = base;
	mem.setCell(base,
		    new LCDControl(base));
	mem.setCell(base + 1,
		    new LCDData(base + 1));
    }
}

