package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the LCD Data device register.
 * <p>
 * This is NOT a singleton!  There are reasons for this
 * choice (e.g. more than one LCD display device); 
 * </p>
 *  @version 1.0 (August 30, 2002)
 *  @author Ken Clowes
 */

public class LCDData
    extends SpecialByteCell
    implements Cell {

    private int contents = 0;
    private MemorySpace mem = MemorySpace64K.getInstance();

    public LCDData(int address)
    {
	super(address);
    }

    final public int get()
    {
	return contents;
    }

    final public int read()
    {
	nReads++;
	cpu.incrementReadCycles();
	return  get();
    }

    public void set(int v)
    {
	contents = v;
    }

    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written to is tracked.
     *
     * @param contents What is written to the byte.
     */
    public void write(int contents)
    {
	LCDControl control = (LCDControl) mem.getCell(address - 1);
	nWrites++;
	cpu.incrementWriteCycles();
	if (control.isBusy()) {
	    Warn.Custom(this,
			"Write to LCD Data register (" +
			Utility.int16ToHex(address) +
			") ignored because device busy.");
	} else {
	    set(contents);
	    control.addCharacter((char) contents);
	    control.setBusy();
	}
    }

    public String toString()
    {
	return "LCDData at 0x" + Utility.int16ToHex(address);
    }

    public String getName()
    {
	return "LCDData";
    }

}

