package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the Output Compare 1 Mask Register (OC1M).
 * <p>
 * The bits in the OC1M register are:
 * <pre>
 *   +-----------------------------------------------+
 *   |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |  Bit numbers
 *   +-----------------------------------------------+
 *    OC1M7 OC1M6 OC1M5 OC1M4 OC1M3                     Mneumonics
 *      0     0     0     0     0     0     0     0     Reset values
 * </pre>
 *
 * <p>
 *  The TOI (Free-running counter overflow) bit enables Timer
 * overflow interrupts.
 * </p>
 *  @version 1.0 (October 15, 2002)
 *  @author Ken Clowes
 */

public class Oc1m
    extends SpecialByteCell
    implements Cell {

    private int contents = 0;
    private static Oc1m INSTANCE = new Oc1m(DevRegisters.OC1M_ADDRESS);
    public static Oc1m getInstance() {return INSTANCE;}
    
    private Oc1m(int address)
    {
	super(address);
    }

    public int get()
    {
	return contents;
    }

    public int read()
    {
	nReads++;
	cpu.incrementReadCycles();
	return  get();
    }

    public void set(int v)
    {
	contents = v&0xF8;
    }

    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written to is tracked.
     *
     * @param contents What is written to the byte.
     */
    public void write(int contents)
    {

	nWrites++;
	cpu.incrementWriteCycles();
	this.contents = contents&0xF3;
    }

    public String getName()
    {
	return "OC1M";
    }
}

