package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;
/**
 * This class implements a byte of independly readable, writeable memory.
 *
 * @version 1.0 (October 22, 2002)
 * @author Ken Clowes
*/
public class RdWrByteCell extends SpecialByteCell
{
    private final static int DEFAULT_CONTENTS = 0x00;
    private int input = DEFAULT_CONTENTS;

    /** Creates a byte of RdWr with known address and initial contents.
     * @param address The address where the byte is mapped.
     * @param contents The initial contents of the byte.
     */

    public RdWrByteCell(int address, int contents)
    {
	super(address, contents);
    }

     /** Creates a byte of RdWr with known address.
      * @param address The address where the byte is mapped.
      */
     public RdWrByteCell(int address)
     {
 	this(address, DEFAULT_CONTENTS);
     }
    /** Returns current input. Can be set with setInput().
     *
     */
    public int read()
    {
	nReads++;
	cpu.incrementReadCycles();
	return  input;
    }

    public void setInput(int i)
    {
	input = i;
    }

    /** Returns a string representation of a RdWrByteCell.
     * The format is:
     * <pre>RdWr@address:contents></pre>
     *
     */
    public String toString()
    {
	return "RdWr@" +
	    Utility.int16ToHex(address) + ":" +
	    Utility.int8ToHex(contents);
    }
    
    public String getName()
    {
	return "Independent Read-Write IO device (RdWr)";
    }
}
