package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;
/**
 * This class implements the internal SCDR (SCI data) device register.
 *
 * @version 1.0 (October 16, 2002)
 * @author Ken Clowes
*/
public class Scdr extends SpecialByteCell
{
    /** Number of cycles to empty Tx Buffer. */
    public final int TX_CYCLES = 100;
    private MemorySpace mem = MemorySpace64K.getInstance();

    private final static int DEFAULT_CONTENTS = 0x00;
    private int txBuffer = DEFAULT_CONTENTS;
    private int rxBuffer = DEFAULT_CONTENTS;
    private Scsr scsr = Scsr.getInstance();

    public static Scdr INSTANCE = new Scdr(DevRegisters.SCDR_ADDRESS,
					DEFAULT_CONTENTS);

    static public Scdr getInstance() {return INSTANCE;}

    private Scdr(int address, int contents)
    {
	super(address, contents);
	mem.setCell(DevRegisters.SCDR_ADDRESS, this);
	reset();
    }

    /** Returns Receive Buffer.
     *
     */
    public int read()
    {
	nReads++;
	cpu.incrementReadCycles();
	scsr.setRxEmpty();
	return  rxBuffer;
    }

    public void setRxBuffer(int c)
    {
	rxBuffer = c;
	scsr.setRxFull();
    }

    public final int getTxBuffer()
    {
	return txBuffer;
    }

    
    public void write(int c)
    {
	if (scsr.isTxEmpty()) {
	    nWrites++;
	    cpu.incrementWriteCycles();
	    scsr.setTxFull();
	    txBuffer = c;
	    cpu.addTimedEvent(new TimedEvent(TX_CYCLES, new TimedAction() {
		    public void doAction()
		    {
			scsr.setTxEmpty();//..BEFORE changing Flag
		    }
		}));
	} else {
	    nWrites++;
	    cpu.incrementWriteCycles();
	    Warn.Custom(this,
			"Write to SCI Data register (" +
			Utility.int16ToHex(address) +
			") ignored; Tx Buffer not Empty.");
	}
    }

    public void reset()
    {
	set(0xD0);
	txBuffer = DEFAULT_CONTENTS;
    }

    /** Returns a string representation of a Scdr.

     */
    public String toString()
    {
	return "SCDR@" +
	    Utility.int16ToHex(address) + ":" +
	    Utility.int8ToHex(contents);
    }
    
    public String getName()
    {
	return "SCDR";
    }
}
