package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;
/**
 * This Singleton class implements the internal SCSR (SCI status)
 * device register.
 *<p>
 * The bits in the SCSR register are:
 * <pre>
 *   +-----------------------------------------------+
 *   |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |  Bit numbers
 *   +-----------------------------------------------+
 *     TDRE  TC    RDRF  IDLE  OR    NF    FE     0     Mneumonics
 *      1     1      0          0     0     0     0     Reset values
 * </pre>
 * </p>
 *
 * @version 1.0 (October 16, 2002)
 * @author Ken Clowes
*/
public class Scsr extends SpecialByteCell
{
    private MemorySpace mem = MemorySpace64K.getInstance();

    private final static int DEFAULT_CONTENTS = 0x00;

    private static Scsr INSTANCE = new Scsr(DevRegisters.SCSR_ADDRESS,
					DEFAULT_CONTENTS);

    static public Scsr getInstance() {return INSTANCE;}

    private Scsr(int address, int contents)
    {
	super(address, contents);
	mem.setCell(DevRegisters.SCSR_ADDRESS, this);
	reset();
    }

    /** Returns contents.
     *
     */
    public int read()
    {
	nReads++;
	cpu.incrementReadCycles();
	return  get();
    }
    
    public void reset()
    {
	set(0xC0);
    }

    /** Returns a string representation of a Scsr.

     */
    public String toString()
    {
	return "SCSR@" +
	    Utility.int16ToHex(address) + ":" +
	    Utility.int8ToHex(contents);
    }
    
    public String getName()
    {
	return "SCSR";
    }

    final public boolean isTxEmpty()
    {
	return ((get()&0x80) == 0x80);
    }

    final public void setTxEmpty()
    {
	set(get()|0x80);
    }

    final public void setTxFull()
    {
	set(get()&0x7F);
    }

    final public void setRxEmpty()
    {
	set(get()&0xDF);
    }

    final public void setRxFull()
    {
	set(get()|0x20);
    }

    final boolean isRxFull()
    {
	return ((get()&0x20) == 0x20);
    }
}
