package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Action to perform on TOC (Timer Output Compare).
 */
class TOCAction implements TimedAction
{
    static private final boolean DEBUG = false;
    //static private final boolean DEBUG = true;
    
    private CPU6811 cpu = CPU6811.getInstance();
    private MemorySpace mem = MemorySpace64K.getInstance();
    private int channel;
    private Tflg1 tflg1 = Tflg1.getInstance();
    private Tctl1 tctl1 = Tctl1.getInstance();
    private PortA portA = PortA.getInstance();

    public TOCAction(int chn)
    {
	channel = chn;
    }

    public void doAction()
    {
	if (DEBUG) {
	    System.out.println(this + " fired");
	}
	//Set TFLG1 bit
	tflg1.setOCFlag(channel);
	doPortA();
    }

    static int[] PORTA_BITS = {
	0x00, //not used
	0x80, //KLUDGE OC1
	0x40, //OC2
	0x20, //OC3
	0x10, //OC4
	0x08 //OC5
    };
	
    /** Update bit in PortA as specified in TCTL1.
     * KLUDGE: code dup'ed in Cforc.java...argh...
     */
    private void doPortA()
    {
	int action;
	action = tctl1.getPortAAction(channel);
	switch (action) {
	case Tctl1.DO_NOTHING:
	    break;
	case Tctl1.TOGGLE:
	    portA.toggleBits(PORTA_BITS[channel]);
	    break;
	case Tctl1.CLEAR:
	    portA.clearBits(PORTA_BITS[channel]);
	    break;
	case Tctl1.SET:
	    portA.setBits(PORTA_BITS[channel]);
	    break;
	}
    }

    public String toString()
    {
	return "TOC" + channel + "Action@" +
	    Utility.int32ToHex(cpu.getNCycles());
    }
}
