package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;
/** Implements the lo byte of a Timer Output Compare register.
 *  @version 1.0 (October 23, 2002)
 *  @author Ken Clowes
 */


public abstract class TOClo
    extends SpecialByteCell
    implements Cell {

    protected final static boolean DEBUG = false;
    //protected final static boolean DEBUG = true;

    protected MemorySpace mem = MemorySpace64K.getInstance();

    public TOClo(int address)
    {
	super(address);
	mem.setCell(address, this);
	reset();
    }


    public String getName()
    {
	return "TOC-lo";
    }

    public void reset()
    {
	set(0xff);
	addTOCtimedEvent(getOCChannel(), 0xFFFF);
    }

    /** Compensates for write cycle of low byte and ASSUMED
     * previous write to high byte. */
    private static final int EXTRA_CYCLES = 1;

    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written to is tracked.
     *
     * @param contents What is written to the byte.
     */
    public void write(int contents)
    {
	super.write(contents);
	int time = (mem.getCell(address-1).get()<<8) + contents;
	
	time = time - (int)cpu.getNCycles() + EXTRA_CYCLES;
	time &= 0xFFFF;
	
	addTOCtimedEvent(getOCChannel(), time);
    }
    
    protected abstract int getOCChannel();

    void addTOCtimedEvent(int channel, int time)
    {
	TimedEvent event = new TimedEvent(time, new TOCAction(channel));
	if (DEBUG) {
	    System.out.println("Adding timed event: " + event);
	}
	cpu.addTimedEvent(event);
    }
}

