package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the Timer Flags 1 (TFLG1) device register.
 * <p>
 * The bits in the TFLG1 register are:
 * <pre>
 *   +-----------------------------------------------+
 *   |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |  Bit numbers
 *   +-----------------------------------------------+
 *    OC1F  OC2F  OC3F  OC4F  OC5F  IC1I IC2I   IC3I    Mneumonics
 *      0    0     0     0      0     0     0     0     Reset values
 * </pre>
 *
 * <p>
 *
 * </p>
 *  @version 1.0 (October 25, 2002)
 *  @author Ken Clowes
 */

public class Tflg1
    extends SpecialByteCell
    implements Cell {

    protected MemorySpace mem = MemorySpace64K.getInstance();
    private static int DEFAULT_CONTENTS = 0x00;
    private int tflg1_contents;
    private static Tflg1 INSTANCE = null;
    public static Tflg1 getInstance()
    {
	if (INSTANCE == null) {
	    INSTANCE = new Tflg1(DevRegisters.TFLG1_ADDRESS);
	}
	return INSTANCE;
    }

    private Tflg1(int address) //Singleton...private constructor
    {
	super(address);
	mem.setCell(address, this);
	reset();
    }


    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written to is tracked. The '1' bits in the contents
     * that are written clear the corresponding bits in the TFLG1
     * device register.
     *
     * @param contents What is written to the byte.
     */
    public void write(int contents)
    {

	int c;
	nWrites++;
	cpu.incrementWriteCycles();
	c = get();
	c = c & (~contents);
	set(c);
    }

    static final int[] OCBitPatterns = {
	0x00, //not used
	0x80, //OC1
	0x40, //OC2
	0x20, //OC3
	0x10, //OC4
	0x08  //OC5
    };

    void setOCFlag(int channel)
    {
	assert((channel >= 1) && (channel <=5));
	int pattern = OCBitPatterns[channel];
	//	contents |= pattern;
	set(get()|pattern);
    }
    
    public String getName()
    {
	return "TFLG1";
    }

    public void reset()
    {
	set(DEFAULT_CONTENTS);
    }

}

