package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the 6811 Timer subsystem.
 *  It creates and maps the special memory cells (i.e. device registers):
 *  <ul>
 *    <li>The free running 16-bit cycle counter (TCNT)
 *    implemented with the {@link TcntHi TcntHi} and
 *    {@link TcntLo TcntLo} classes.</li>
 *    <li>Timer flags 2 (TFLG2) implemented with the
 *    {@link Tflg2 Tflg2} class.</li>
 *  </ul>
 *
 *  @version 1.0 (August 11, 2002)
 *  @author Ken Clowes
 */

public class Timer {
    public static final int REG_BAS = DevRegisters.REG_BAS;
    public static final int TCNT = DevRegisters.TCNT;
    public static final int TCNT_HI_ADDRESS = DevRegisters.TCNT_HI_ADDRESS;
    public static final int TCNT_LO_ADDRESS = DevRegisters.TCNT_LO_ADDRESS;
    public static final int TFLG2 = DevRegisters.TFLG2;
    public static final int TFLG2_ADDRESS = DevRegisters.TFLG2_ADDRESS;
    public static final int TMSK2 = DevRegisters.TMSK2;
    public static final int TMSK2_ADDRESS = DevRegisters.TMSK2_ADDRESS;

    private MemorySpace mem = MemorySpace64K.getInstance();
    private Cell tflg2Cell = null;
    static private Cell tmsk2Cell = Tmsk2.getInstance();
    static private Cell oc1mCell = Oc1m.getInstance();
    private CPU6811 cpu = CPU6811.getInstance();

    public Timer()
    {
	mem.setCell(TCNT_HI_ADDRESS,
		    new TcntHi(TCNT_HI_ADDRESS));

 	mem.setCell(TCNT_LO_ADDRESS,
 		    new TcntLo(TCNT_LO_ADDRESS));

 	mem.setCell(TFLG2_ADDRESS,
 		    tflg2Cell = new Tflg2(TFLG2_ADDRESS));
	
	mem.setCell(TMSK2_ADDRESS,
		    tmsk2Cell = Tmsk2.getInstance());
	
	cpu.addTimedEvent(new TimedEvent(0x10000, new TOFAction()));

	mem.setCell(DevRegisters.OC1M_ADDRESS,
		    oc1mCell = Oc1m.getInstance());
	

	Tctl1.getInstance();

  	TOC1hi.getInstance();
  	TOC1lo.getInstance();

 	TOC2hi.getInstance();
 	TOC2lo.getInstance();

 	TOC3hi.getInstance();
 	TOC3lo.getInstance();

 	TOC4hi.getInstance();
 	TOC4lo.getInstance();

 	TOC5hi.getInstance();
 	TOC5lo.getInstance();

	Tflg1.getInstance();

	Cforc.getInstance();
    }

}

