package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the Timer Mask 2 (TMSK2) device register.
 * <p>
 * The bits in the TMSK2 register are:
 * <pre>
 *   +-----------------------------------------------+
 *   |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |  Bit numbers
 *   +-----------------------------------------------+
 *     TOI                                              Mneumonics
 *      0                                               Reset values
 * </pre>
 *
 * <p>
 *  The TOI (Free-running counter overflow) bit enables Timer
 * overflow interrupts.
 * </p>
 *  @version 1.0 (August 16, 2002)
 *  @author Ken Clowes
 */

public class Tmsk2
    extends SpecialByteCell
    implements Cell {

    private int contents = 0;
    private static Tmsk2 INSTANCE = new Tmsk2(DevRegisters.TMSK2_ADDRESS);
    public static Tmsk2 getInstance() {return INSTANCE;}
    
    private Tmsk2(int address)
    {
	super(address);
    }

    public int get()
    {
	return contents;
    }

    public int read()
    {
	nReads++;
	cpu.incrementReadCycles();
	return  get();
    }

    public void set(int v)
    {
	contents = v&0xF3;
    }

    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written to is tracked.
     *
     * @param contents What is written to the byte.
     */
    public void write(int contents)
    {

	nWrites++;
	cpu.incrementWriteCycles();
	this.contents = contents&0xF3;
    }
    
    public String getName()
    {
	return "TMSK2";
    }

}

