package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/**
 * This class implements the contents of an address in MemorySpace
 * that has not been mapped to any implemented byte.  It is like
 * an Unused cell, except that warnings are not issued.
 *
 * @version 1.0 (October 29, 2002)
 * @author Ken Clowes
*/

public class UnimplementedByteCell extends AbstractByteCell
{
    private final static int DEFAULT_CONTENTS = 0x01;

    /** Creates a cell with known address.
     * @param address The address where the byte is mapped.
     */


    public UnimplementedByteCell(int address)
    {
	super(address);
	this.address = address;
	contents = DEFAULT_CONTENTS;
	nWrites = 0;
	nReads = 0;
    }

    /** Returns the contents of the byte.  The number of times the
     * cell is read is tracked.
     */
    public int read()
    {
	nReads++;
	return  contents;
    }

    /** Simulates a write bus cycle to the unmapped location.
     * The number of times the byte is written to is tracked.
     * The contents of the unmapped location will not be changed (obviously!)
     *
     * @param contents <b>Ignored</b>
     */

    public void write(int contents)
    {
	nWrites++;
	cpu.incrementWriteCycles();
    }

    /** Simulates a write bus cycle to the unmapped location.
     * The number of times the byte is written to is tracked.
     * The contents of the unmapped location will not be changed (obviously!)
     *
     * @param contents <b>Ignored</b>
     */
    public void write(byte contents)
    {
	write((int) contents);
    }

    /** Returns the contents of the unmapped byte.  There is no tracking of the
     * number of times the cell contents are obtained.
     */
    public int get()
    {
	return  contents;
    }

    /** Setting an unimplemented location does nothing. */
    public void set(int contents)
    {
    }

    public String getName()
    {
	return "NOT IMPLEMENTED";
    }

}
