package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;
/**
 * This class implements a byte of Write Only Memory (WOM).
 *
 * @version 1.0 (September 30, 2002)
 * @author Ken Clowes
*/
public class WOMByteCell extends SpecialByteCell
{
    private final static int DEFAULT_CONTENTS = 0x00;

    /** Creates a byte of WOM with known address and initial contents.
     * @param address The address where the byte is mapped.
     * @param contents The initial contents of the byte.
     */

    public WOMByteCell(int address, int contents)
    {
	super(address, contents);
    }

     /** Creates a byte of WOM with known address.
      * @param address The address where the byte is mapped.
      */
     public WOMByteCell(int address)
     {
 	this(address, DEFAULT_CONTENTS);
     }
    /** Returns 0; contents cannot be read.
     *
     */
    public int read()
    {
	Warn.WOMRead((Cell) this);
	nReads++;
	cpu.incrementReadCycles();
	return  0;
    }
    

    /** Returns a string representation of a WOMByteCell.
     * The format is:
     * <pre>WOM@address:contents></pre>
     *
     */
    public String toString()
    {
	return "WOM@" +
	    Utility.int16ToHex(address) + ":" +
	    Utility.int8ToHex(contents);
    }
    
    public String getName()
    {
	return "Write-Only Memory (WOM)";
    }
}
