package com.kclowes.sim11.ui;
import com.kclowes.sim11.*;
import com.kclowes.sim11.devices.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
/** Creates a frame for 8 channels of analog voltages.
 *
 * <p>
 *  The frame has the following format:
 *<pre>
 *  +---------------------------------------------------+
 *  |              TITLE                                |
 *  +---------------------------------------------------+
 *  +---------------------------------------------------+
 *  |                   channel 1                       |
 *  +---------------------------------------------------+
 *  +---------------------------------------------------+
 *  |                   channel 2                       |
 *  +---------------------------------------------------+
 *  +---------------------------------------------------+
 *                          .
 *                          .
 *                          .
 *  +---------------------------------------------------+
 *  +---------------------------------------------------+
 *  |                   channel 8                       |
 *  +---------------------------------------------------+
 *  +---------------------------------------------------+
 *</pre>
 * </p>
 *  @version 1.0 (September 22, 2002)
 *  @author Ken Clowes
 */

public class ADCui extends JFrame
    implements WindowListener, VoltageGetter
{
    static final boolean DEBUG = false;
    private int[] voltages = new int[8];
    private MemorySpace mem = MemorySpace64K.getInstance();
    private JSlider[] sliders = new JSlider[8];
    private PortE portE = PortE.getInstance();
    /** Creates the basic graphical user interface for
     * setting analog voltages.
     */
    ADCui()
    {
	setTitle("Analog voltages");
	
	setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        getContentPane().add(createPanel());
	addWindowListener(this);
	pack();
	portE.addVoltageChangeListener(new VoltageChangeListener() {
		public void perform(PortE pe)
		{
		    int [] v;
		    if (DEBUG) {
			System.out.println("ADC VChngLstnr: PortE change!");
		    }
		    v = portE.getVoltages();
		    boolean change = false;
		    for(int i = 0; i < 8; i++) {
			if (v[i] != voltages[i]) {
			    change = true;
			    sliders[i].setValue(v[i]);
			}
		    }
		}
	    });

	setLocation(950, 300);
	setVisible(true);
    }
    
    /** Returns an array of ints corresponding to the current position
     * of each JSlider.
     */
    public int[] getVoltages()
    {
	for(int i = 0; i < 8; i++) {
	    voltages[i] = sliders[i].getValue();
	}
	return voltages;
    }
    
    JComponent createPanel()
    {
	JComponent panel = Box.createVerticalBox();
	for(int i = 0; i < 8; i++) {
	    JPanel channelPanel = new JPanel(new BorderLayout());
	    sliders[i] = new JSlider(JSlider.HORIZONTAL, 0, 255, 100);
	    JLabel name = new JLabel("Channel " + (i+1));
	    sliders[i].addChangeListener( new ChangeListener() {
		    public void stateChanged(ChangeEvent e)
		    {
			JSlider source = (JSlider)e.getSource();
			if (!source.getValueIsAdjusting()) {
			    getVoltages();
			    portE.setVoltages(voltages);
			}
		    }
		});
	    channelPanel.add(name, BorderLayout.WEST);
	    channelPanel.add(sliders[i], BorderLayout.EAST);
	    panel.add(channelPanel);
	}
	return panel;
    }
    
    
    public void windowClosing(WindowEvent e)
    {
	System.out.println("Closing ADCui frame.");
    }
    
    public void windowOpened(WindowEvent e)
    {
    }
    public void windowClosed(WindowEvent e)
    {
    }
    public void windowIconified(WindowEvent e)
    {
    }
    public void windowDeiconified(WindowEvent e)
    {
    }
    public void windowActivated(WindowEvent e)
    {
    }
    public void windowDeactivated(WindowEvent e)
    {
    }
    
    /** Test main frame (stand-alone).
     */
    static public void main(String args[])
    {
	ADCui mainFrame = new ADCui();
    }
}
