package com.kclowes.sim11.ui;
//package com.kclowes.sim11.ui;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import com.kclowes.sim11.*;
import java.io.*;

/**
 * Some simple tests.
 * Modified October 7, 2003: test files in std place.
 */
public class DummyUITest extends TestCase {
    /* TEST_DATA_DIR relative to project home */
    String TEST_DATA_DIR="./etc/test-files/";
    private MemorySpace mem = MemorySpace64K.getInstance();
    private CPU6811 cpu = CPU6811.getInstance();
    private AbstractUI ui = new DummyUI();
    private LstLoader lstLoader = LstLoader.getInstance();

    public DummyUITest(String name) {
	super(name);
    }
    
    protected void setUp() {
	mem.init();
	ui.setConfig();
    }
    
    public static Test suite() {
	
	TestSuite suite= new TestSuite();
	suite.addTest(new DummyUITest("testOpenS19File"));
	suite.addTest(new DummyUITest("testOpenLSTFile"));
	suite.addTest(new DummyUITest("testOpenLSTFile2"));
	suite.addTest(new DummyUITest("testOpenLSTFile3"));
	suite.addTest(new DummyUITest("testOpenLSTFile4"));
	return suite;
    }
    
    /**
     * Primitive test to open an object (<code>*.s19</code> file.
     */
    public void testOpenS19File() {
	try {
	    ui.loadS19(TEST_DATA_DIR + "foo.s19");
	} catch  (IOException exception) {
	    System.err.println(exception);
	}
	Cell cell = mem.getCell(0x6000);
	int ad = cell.getAddress();
	int v = cell.get();
	assertEquals(ad, 0x6000);
	assertEquals(v, 0xB6);
	
    }

    /**
     * Primitive test to open a source (<code>*.lst</code>) file.
     */
    public void testOpenLSTFile() {
	String line = null;
	String shouldBe = null;
	SourceLine[] lines = null;
	try {
	    lines = ui.loadLST(TEST_DATA_DIR + "foo.lst");
	} catch  (IOException exception) {
	    System.err.println(exception);
	} catch (Exception e) {
	    System.err.println(e);
	}
	assertEquals(9, lines.length);
	for(int i = 0; i < 9; i++) {
	    assertEquals(i+1, lines[i].lineNumber);
	    line = lines[i].sourceCode;
	    switch (i) {
	    case 0:
		shouldBe = "       org $6000";
		assertEquals(shouldBe, line);
		assertEquals(SourceLine.Type.OTHER, lines[i].type);
		break;
	    case 1:
		shouldBe = "[ 4 ]  ldaa countLongVarName";
		assertEquals(shouldBe, line);
		assertEquals(SourceLine.Type.INSTRUCTION, lines[i].type);
		break;
	    default:
		break;
	    }
	}
    }

    /**
     * Primitive test to open a second source (<code>*.lst</code>) file.
     */
    public void testOpenLSTFile2() {
	try {
	    lstLoader.loadLST(TEST_DATA_DIR + "foo7000.lst");
	    File [] srcFiles =  lstLoader.getLoadedSrcFiles();
	} catch (IOException exception) {
	    System.err.println(exception);
	}
	//Now two files open...
	int nFiles = lstLoader.getLoadedSrcFiles().length;
	assertEquals(2, nFiles);
    }

    /**
     * Primitive test to ensure that file cannot be added twice.
     */
    public void testOpenLSTFile3() {
	try {
	    lstLoader.loadLST(TEST_DATA_DIR + "foo7000.lst");
	    assertTrue(false);
	} catch (NoDuplicateFileException exception) {
	    System.err.println(exception);
	} catch (IOException exception) {
	    System.err.println(exception);
	}
	//Still two files open...
	int nFiles = lstLoader.getLoadedSrcFiles().length;
	assertEquals(2, nFiles);
    }

    /**
     * Primitive test for unloading files.
     */
    public void testOpenLSTFile4() {
	File fileToUnload = new File(TEST_DATA_DIR + "foo7000.lst");
	try {
	    lstLoader.unload(fileToUnload);
	} catch (Exception exception) {
	    System.err.println(exception);
	}
	//Now only one file open...
	int nFiles = lstLoader.getLoadedSrcFiles().length;
	assertEquals(1, nFiles);
    }
}
