package com.kclowes.sim11.ui;
import com.kclowes.sim11.*;
import com.kclowes.sim11.devices.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
/** Creates a frame for LCD graphical user interface.
 * <p>
 *  The frame has the following format:
 *<pre>
 *  +---------------------------------------------------+
 *  |              TITLE                                |
 *  +---------------------------------------------------+
 *  +---------------------------------------------------+
 *  |               LCD LINE 1                          |
 *  +---------------------------------------------------+
 *  +---------------------------------------------------+
 *  |               LCD LINE 2                          |
 *  +---------------------------------------------------+
 *  +---------------------------------------------------+
 *  |               LCD STATUS                          |
 *  +---------------------------------------------------+
 *</pre>
 * </p>
 *  @version 1.0 (September 2, 2002)
 *  @author Ken Clowes
 */

public class LCDui extends JFrame implements WindowListener
{
    private int address;//Address of control register
    private LCD lcdDevice = LCD.getInstance();
    private JLabel status; //Displays whether the device is BUSY/READY
    private JComponent lcd;   //The actual LCD display (2 lines)
    private MemorySpace mem = MemorySpace64K.getInstance();
    private LCDControl cellLCDControl;
    private JLabel line1;
    private JLabel line2;
    private char lines[][];

    //Fonts etc. for status info
    static int STATUS_FONT_SIZE = 10;
    static String STATUS_FONT_NAME = "SansSerif";
    static int STATUS_FONT_STYLE = Font.BOLD;
    static Font STATUS_FONT = new Font(STATUS_FONT_NAME,
				      STATUS_FONT_STYLE,
				      STATUS_FONT_SIZE);
    static Color STATUS_FOREGROUND = Color.BLACK;
    static Color STATUS_BACKGROUND = Color.GRAY;

    //Fonts etc. for LCD display
    static int LCD_FONT_SIZE = 20;
    static String LCD_FONT_NAME = "Monospaced";
    static int LCD_FONT_STYLE = Font.BOLD;
    static Font LCD_FONT = new Font(LCD_FONT_NAME,
				      LCD_FONT_STYLE,
				      LCD_FONT_SIZE);
    static Color LCD_FOREGROUND = Color.BLACK;
    static Color LCD_BACKGROUND = Color.GRAY;

    /** Create a Graphical User Interface for an LCD device.
     */
    LCDui(int baseAddress)
    {
	address = baseAddress;
	setTitle("LCD@0x" + Utility.int16ToHex(address) +
		 "-" +
		 Utility.int16ToHex(address+1) +
		 "(ctr/data)");
	setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        getContentPane().add(createLCDPanel());
	addWindowListener(this);
	pack();
	setLocation(950, 100);
	setVisible(true);
 	cellLCDControl = (LCDControl) mem.getCell(address);

 	cellLCDControl.addCellChangeListener(new CellChangeListener() {
 		public void perform(Cell c)
 		{
		    status.setText(cellLCDControl.isBusy() ?
				   "BUSY" : "READY");
		    setLines();
 		}
 	    });
    }

    JComponent createLCDPanel()
    {
	JPanel panel = new JPanel(new BorderLayout());
	status = createStatusLabel();
	panel.add(status, BorderLayout.SOUTH);

	lcd = createLCDPane();
	panel.add(lcd, BorderLayout.NORTH);
	return panel;
    }
    
    private JLabel createStatusLabel()
    {
	JLabel status = new JLabel("READY");
	status.setFont(STATUS_FONT);
	status.setOpaque(true);
	status.setBackground(STATUS_BACKGROUND);
	status.setForeground(STATUS_FOREGROUND);
	return status;
    }

    private JComponent createLCDPane()
    {
	JComponent lcd = Box.createVerticalBox();
	line1 = new JLabel("                    ");
	line1.setFont(LCD_FONT);
	line1.setOpaque(true);
	line1.setBackground(LCD_BACKGROUND);
	line1.setForeground(LCD_FOREGROUND);
	lcd.add(line1);
	line2 = new JLabel("                    ");
	line2.setFont(LCD_FONT);
	line2.setOpaque(true);
	line2.setBackground(LCD_BACKGROUND);
	line2.setForeground(LCD_FOREGROUND);
	lcd.add(line2);
	//	setLines();
	return (JComponent) lcd;
    }

    private void setLines()
    {
	lines = cellLCDControl.getLines();
	line1.setText(new String(lines[0]));
	line2.setText(new String(lines[1]));
    }
    

    public void windowClosing(WindowEvent e)
    {
	System.out.println("Closing LCDui frame.");
    }

    public void windowOpened(WindowEvent e)
    {
    }
    public void windowClosed(WindowEvent e)
    {
    }
    public void windowIconified(WindowEvent e)
    {
    }
    public void windowDeiconified(WindowEvent e)
    {
    }
    public void windowActivated(WindowEvent e)
    {
    }
    public void windowDeactivated(WindowEvent e)
    {
    }
    static public void main(String args[])
    {
	LCDui mainFrame = new LCDui(0x1400);
    }
}
