package com.kclowes.sim11.ui;
import java.io.*;
import java.util.*;
import java.util.regex.*;

/** Loads and unloads <code>*.lst</code>files.
 *
 * <p>
 * The class tracks all the loaded source files.  It can:
 *
 * </p>
 *  @version 1.0 (October 9, 2002)
 *  @author Ken Clowes
 */

public class LstLoader
{
    private static LstLoader INSTANCE = new LstLoader();
    private ArrayList loadedSrc = new ArrayList();
    private ArrayList srcLinesArrayList = new ArrayList();
    SourceLine[] srcLines = null;
    HashMap PCtoSrcLineNum = new HashMap();

    private LstLoader()
    {
    }

    public static LstLoader getInstance() {
	return INSTANCE;
    }
    /** Parses a listing file, returning an array of <code>SourceLine</code>.
     *
     * @param fileName File to load/parse
     * @throws NoLSTFileException
     */
    public SourceLine[] loadLST(String fileName) throws NoLSTFileException,
	NoDuplicateFileException
    {
	SourceLine[] result = null;
	File file = new File(fileName);
	if (loadedSrc.contains(file)) {
	    throw new NoDuplicateFileException(fileName);
	}
	try {
	    
	    Reader reader = new FileReader(fileName);
	    result = loadLST(file, reader);
	    reader.close();
	    loadedSrc.add(file);
	} catch (FileNotFoundException e) {
	    throw new NoLSTFileException(fileName);
	} catch (IOException e) {
	    System.out.println("Got unexpected exception: " + e);
	}
	return result;
	    
    }

    
    /** Get an array of loaded source Files.
     */
    public File[] getLoadedSrcFiles()
    {
	return (File[]) loadedSrc.toArray(new File[0]);
    }
    
    
    /** Parses a listing file, returning an array of <code>SourceLine</code>.
     *
     * @param reader 
     * @throws IOException
     */

    private SourceLine[] loadLST(File file, Reader reader) throws IOException
    {
	// KLUDGE the 31 magic number is VERY specific to list format!!!
	Pattern numberedLine = Pattern.compile("(^\\d{4}).{25}(.*)");
	Pattern sourceLinePat =
	    Pattern.
	    compile(
		    "^\\d{4} ([0-9A-Fa-f]{4}).{18,26}\\S*\\s+(\\S*).*");
	Pattern sourceLinePat2 =
	   Pattern.compile("^\\d{4} ([0-9A-Fa-f]{4}).{20}\\S*\\s+(\\S*).*");
	Matcher m;
	String str;

	BufferedReader bf = new BufferedReader(reader);
	bf.readLine();//Ignore first line
	while ((str = bf.readLine()) != null) {
	    m = numberedLine.matcher(str);
	    if (m.matches()) {
		SourceLine sourceLine = new SourceLine();
		String srcLine = m.group(2);
		int lineNum = Integer.parseInt(m.group(1));
		m = sourceLinePat.matcher(str);
		boolean isInst = false;
		int pc = 0;
		m = sourceLinePat.matcher(str);
		if (m.matches()) {
		    String op = m.group(2);
		    String pcS = m.group(1);
		    pc = Integer.parseInt(pcS, 16);
		    if (isInstruction(op)) {
			isInst = true;
		    } else {
			m = sourceLinePat2.matcher(str);
			if (m.matches()) {
			    op = m.group(2);
			    pcS = m.group(1);
			    pc = Integer.parseInt(pcS, 16);
			    if (isInstruction(op)) {
				isInst = true;
			    }
			}
		    }
		}
		sourceLine.file = file;
		sourceLine.lineNumber = lineNum;
		sourceLine.sourceCode = srcLine;
		sourceLine.type = SourceLine.Type.OTHER;
		sourceLine.PC = -1;
		if (isInst) {
		    sourceLine.type = SourceLine.Type.INSTRUCTION;
		    sourceLine.PC = pc;
		    PCtoSrcLineNum.put(new Integer(pc), new Integer(lineNum));
		}
		srcLinesArrayList.add(sourceLine);
	    }
	}
	srcLines = (SourceLine[]) srcLinesArrayList.toArray(new SourceLine[0]);
	return srcLines;
    }

    /** Unloads the specified source code file. */
    public SourceLine[] unload(File file)
    {
	for(Iterator iter = srcLinesArrayList.iterator();
	    iter.hasNext();) {
	    SourceLine line = (SourceLine) (iter.next());
	    if (line.file == file) {
		iter.remove();
	    }
	}
	loadedSrc.remove(file);
	return (SourceLine[]) srcLinesArrayList.toArray(new SourceLine[0]);
    }

    private boolean isInstruction(String op)
    {
	return Mnemonics.instructions.contains(op.toLowerCase());
    }

}

