package com.kclowes.sim11.ui;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;

/** The Message Window pane.  Dedicated to sim11 Messages pane for now.
 * (This is a kludge...to be fixed some day...)
 * <p>
 * As messages are added, they are displayed without scrolling if the
 * added message fits into the current View.  Otherwise, the View is
 * automatically scrolled vertically to place the added message at
 * (about) the middle of the window.
 * </p>
 *  @version 1.0 (October 3, 2002)
 *  @author Ken Clowes
 */
//Should probably be a Singleton..

public class Messages extends JTextPane
{
    static final Dimension TRACE_PREF = new Dimension(650, 300);
    
    private final int N_BLANK_LINES = 10;
    private final String BLANK_LINE = " \n";
    private final int BLANK_LINE_LENGTH = 2;

    // Color/font constants
    
    static final Color LEGAL_COLOR = Color.RED;
    static final boolean LEGAL_ITALIC = false;
    static final boolean LEGAL_BOLD = false;
    static final int LEGAL_SIZE = 10;
    
    private Hashtable styleAttributes;
    private StyleContext sc = new StyleContext();
    private DefaultStyledDocument styledDoc = new DefaultStyledDocument(sc);
    private Document traceMsgDoc;
    private JScrollPane traceMsgScroll;
    
    Messages() {
	//Create document styles ...from GUTZ page 193
	super();
	createStyles(sc);
 	traceMsgDoc = getDocument();
	traceMsgScroll = new JScrollPane(this);
	traceMsgScroll.setPreferredSize(TRACE_PREF);
	String title = "Messages and Traces";
	traceMsgScroll.setBorder(
				 BorderFactory.createTitledBorder(title));
	Style s = (Style) styleAttributes.get("message");
	try {
	    for(int i = 0; i < N_BLANK_LINES; i++) {
		traceMsgDoc.insertString(traceMsgDoc.getLength(),
					 BLANK_LINE, s);
	    }
	} catch (BadLocationException badLocation) {
	    System.err.println("Couldn't insert BLANK lines text.");
	    System.exit(1);//KLUDGE
	}

    }
    
    public JScrollPane getScrollable()
    {
	return traceMsgScroll;
    }
    
    private void createStyles(StyleContext sc)
    {
	//from GUTZ (should figure out what it means!
	Style style;
	styleAttributes = new Hashtable();
	
	//no style
	style = sc.addStyle(null, null);
	styleAttributes.put("none", style);
	
	//message
	style = sc.addStyle(null, null);
	StyleConstants.setForeground(style, Color.BLUE);
	styleAttributes.put("message", style);
	
	//legal
	style = sc.addStyle(null, null);
	StyleConstants.setForeground(style, LEGAL_COLOR);
	StyleConstants.setItalic(style, LEGAL_ITALIC);
	StyleConstants.setBold(style, LEGAL_BOLD);
	StyleConstants.setFontSize(style, LEGAL_SIZE);
	styleAttributes.put("legal", style);
	
	//trace
	style = sc.addStyle(null, null);
	StyleConstants.setForeground(style, Color.BLACK);
	StyleConstants.setFontFamily(style, "Courier");
	styleAttributes.put("trace", style);
	
	//UIwarning
	style = sc.addStyle(null, null);
	StyleConstants.setForeground(style, Color.RED);
	StyleConstants.setItalic(style, true);
	styleAttributes.put("UIwarning", style);
    }
    
    /** Add a message to the Messages pane using the
     * default "message" style.  The message is appended
     * to the window.
     *
     * <p>Note that it is the programmer's
     * responsibility to separate messages into lines; this usually
     * means that messages should be terminated with a newline
     * (<code>'\n'</code>) character.
     * </p>
     *
     * @param msg message to add.
     */
    public void showMessage(String msg)
    {
	showMessage(msg, "message");
    }
    
    /** Add a styled message to the traceMsg window.  The message is appended
     * to the traceMsg window.
     *
     * <p>Note that it is the programmer's
     * responsibility to separate messages into lines; this usually
     * means that messages should be terminated with a newline
     * (<code>'\n'</code>) character.
     * </p>
     *
     * @param msg message to add.
     * @param style description of style
     */
    public void showMessage(String msg, String style)
    {
	int realHeight = getHeight();
	JViewport viewport = traceMsgScroll.getViewport();
	Point viewPosition = viewport.getViewPosition();
	int viewHeight = viewPosition.y;
	int magic = 250;  //Sorry...but this KLUDGE seems to work (sort of...)
	int magic2 = 440;
	if (viewHeight + magic2 < realHeight) {
	    viewPosition.y += magic;
	    while(viewPosition.y + magic < realHeight) {
		viewPosition.y += magic;
	    }
 	    traceMsgScroll.getViewport().setViewPosition(viewPosition);
	}
	try {
	    Style s = (Style) styleAttributes.get(style);
	    traceMsgDoc.insertString(traceMsgDoc.getLength() -
				     N_BLANK_LINES*BLANK_LINE_LENGTH,
				     msg, s);
	} catch (BadLocationException ble) {
	    System.err.println("Couldn't insert text.");
	    System.exit(1);//KLUDGE
	}
    }
}

