package com.kclowes.sim11.ui;
/**
 * Basic parameters used by the virtual robot device.
 * (NOTE: this is a <em>detail</em> class...)
 * <p>
 * The simulation of the eeBot's position depends on obtaining
 * information such as:
 * <dl>
 * <li>What distance has each wheel travelled?</li>
 * <li>What is the distance between the wheels?</li>
 * </dl>
 * </p>
 *
 * <p>
 * In principle, any client interested in a change in a motor's
 * rotation  could request notification at every bus cycle.  This
 * approach is "doable" (i.e. add recursive timed events to every
 * clock cycle), but 
 */
public class RobotParameters
{

     /** Number of motor ticks (either motor) required to force
     * position recalculation.
     */
    public static final int RECALCULATE_POSITION = 50;
    
    /** Difference if left/right ticks to force
     * orientation recalculation.
     */
    public static final int RECALCULATE_ORIENTATION = 10;
    
    /**
     * Number of bus cycles before modifing motor ticks
     */
    public static final int CYCLES_PER_TICK = 1000;

    /**
     * Speed of eeBot in metres per second.
     */
    public static final double SPEED = 0.1;
    
    /**
     * Distance between wheels in metres.
     */
    public static final double DIAMETER = 0.08;
    
    /**
     * Number of motor ticks per metre.
     */
    public static final double TICKS_PER_METER = 10000;
    


    //Class cannot be instantiated.
    private RobotParameters() {}




//     //Motor ticks per second
//     private static final double TICKS_PER_SECOND =
// 	0.000001*MONITOR_TICKS_CYCLES;  //1 MHz E-clock

//     private static final double BOT_SPEED = 0.1; // metres/sec
//     private static final double BOT_DIAMETER = 0.08; // metres
//     private static final double BOT_RADIUS = BOT_DIAMETER/2.0; // metres
//     private static final double TICKS_PER_METER =
// 	1/(BOT_SPEED*TICKS_PER_SECOND);
//     //Number of ticks before recalculating robot position
//     static final private int RECALCULATE_POSITION =
// 	(int) (TICKS_PER_METER/1000.0 + .5);
//     //Difference in left/right ticks before recalculating robot orientation
//     //Set to 1% of BOT_DIAMETER
//     static final private int RECALCULATE_ORIENTATION =
// 	RECALCULATE_POSITION/4;
// 	//	(int) ((0.01)*(TICKS_PER_METER/(1000*BOT_DIAMETER)));


}
