package com.kclowes.sim11.ui;
import com.kclowes.sim11.*;
import com.kclowes.sim11.devices.*;
import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
/** Creates a vitual terminal to communicate with the SCI subsystem.
 * <p>
 *  The frame has the following format:
 *<pre>
 *  +---------------------------------------------------+
 *  |              TITLE                                |
 *  +---------------------------------------------------+
 *  |                                                   |
 *  +---------------------------------------------------+
 *</pre>
 * </p>
 *  @version 1.0 (October 18, 2002)
 *  @author Ken Clowes
 */

public class SCIui extends JFrame
    implements WindowListener
{
    private Scsr scsr = Scsr.getInstance();
    private Scdr scdr = Scdr.getInstance();
    private JTextArea terminal;
    private JScrollPane terminalScrollPane;
    //Fonts etc. for SCI display
    static int SCI_FONT_SIZE = 12;
    static String SCI_FONT_NAME = "Monospaced";
    static int SCI_FONT_STYLE = Font.PLAIN;
    static Font SCI_FONT = new Font(SCI_FONT_NAME,
				      SCI_FONT_STYLE,
				      SCI_FONT_SIZE);
    static Color SCI_FOREGROUND = Color.BLACK;
    static Color SCI_BACKGROUND = Color.WHITE;

    private final int N_BLANK_LINES = 10;
    private final String BLANK_LINE = " \n";
    private final int BLANK_LINE_LENGTH = 2;


//     private SCIui INSTANCE = new SCIui();
//     public SCIui getInstance() {return INSTANCE;}

    /** Creates the basic graphical user interface for
     * the SCI (serial port).
     */
    public SCIui() //Singleton, Constructor private NOT!
    {
	setTitle("SCI serial port");

	setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        getContentPane().add(createPanel());

	terminal.setEditable(false);
	for(int i = 0; i < N_BLANK_LINES; i++) {
	    terminal.append(BLANK_LINE);
	}

	terminal.addKeyListener(new KeyListener() {
		//KLUDGE explanation...
		//I wrote this ghastly code while attempting to get the
		//Buffalo Monitor to work...It needs to read raw ASCII
		//INCLUDING things like CR and NL and it NEEDS to get
		//raw CR's... But even "keyTyped().getKeyCode()" give
		//a 0x0A (newline) character when I hit the ENTER key...
		//This kludge (YES, I know there must be some more elegant
		//way around this problem...) "solves" the problem...
		//(sort of...)...
		boolean lastWasCR = false;  //KLUDGE
		public void keyPressed(KeyEvent e)
		{
		    lastWasCR = (e.getKeyCode() == KeyEvent.VK_ENTER);
		}
		public void keyReleased(KeyEvent e) {}
		public void keyTyped(KeyEvent e) {
		    char ch;
		    ch = e.getKeyChar();
		    if (lastWasCR) {
			ch = 0x0D;
		    }
		    scdr.setRxBuffer(ch);
		}
	    });
 	scsr.addCellChangeListener(new CellChangeListener() {
		private boolean TxWasFull = true;
 		public void perform(Cell c)
 		{
		    assert (c == scsr);

		    if (scsr.isTxEmpty()) {
			int realHeight = terminal.getHeight();
			JViewport viewport = terminalScrollPane.getViewport();
			Point viewPosition = viewport.getViewPosition();
			int viewHeight = viewPosition.y;
			int magic = 250;  //Sorry...but this KLUDGE 
			int magic2 = 440;
			if (viewHeight + magic2 < realHeight) {
			    viewPosition.y += magic;
			    while(viewPosition.y + magic < realHeight) {
				viewPosition.y += magic;
			    }
			    terminalScrollPane.
				getViewport().setViewPosition(viewPosition);
			}
			if (TxWasFull) {
			    TxWasFull = false;
			    char ch = (char) scdr.getTxBuffer();
			    if (ch != 0) {
				char charArray[] = {ch};
				String toInsert = new String(charArray);
				Document doc = terminal.getDocument();
				terminal.insert(toInsert,
						doc.getLength() -
						N_BLANK_LINES*
						BLANK_LINE_LENGTH);
				//terminal.append(new String(charArray));
			    }
			}
		    } else {
			TxWasFull = true;
		    }
 		}
 	    });
	
	addWindowListener(this);
 	pack();
 	setLocation(750, 500);
	setVisible(true);
    }
    
    
    JComponent createPanel()
    {
	terminal =new JTextArea(24, 80);
	terminal.setFont(SCI_FONT);
	terminalScrollPane = new JScrollPane(terminal);
	return terminalScrollPane;
    }
    
    public void windowClosing(WindowEvent e)
    {
	System.out.println("Closing SCIui frame.");
    }
    
    public void windowOpened(WindowEvent e)
    {
    }
    public void windowClosed(WindowEvent e)
    {
    }
    public void windowIconified(WindowEvent e)
    {
    }
    public void windowDeiconified(WindowEvent e)
    {
    }
    public void windowActivated(WindowEvent e)
    {
    }
    public void windowDeactivated(WindowEvent e)
    {
    }
    
    /** Test main frame (stand-alone).
     */
    static public void main(String args[])
    {
	SCIui mainFrame = new SCIui();
    }
}
