package com.kclowes.sim11.ui;
import com.kclowes.sim11.*;
import com.kclowes.sim11.devices.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;
import javax.swing.table.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

/** The basic Swing graphical interface for the simulator.
 *  @version 0.97 (October 22, 2002)
 *  @author Ken Clowes
 */
//Probably should be a Singleton?

public class SwingUI extends AbstractUI
{
    //Many of these constants may be better placed in preferences.

    //Constants for placement of frame
    static final int FRAME_UPPER_X = 10;
    static final int FRAME_UPPER_Y = 15;

    // Dimension constants (windows)
    static final Dimension STATS_PREF = new Dimension(250, 130);
    static final Dimension REGS_PREF = new Dimension(250, 200);

    static final Dimension SRC_PREF = new Dimension(650, 300);

    String SIM11_HOME = System.getProperty("SIM11_HOME");
    //GARBAGE static String SIM11_HOME = "C:\\home\\kclowes\\java\\sim11";

    private JFrame frame;
    
    private Messages traceMsg = new Messages();  //Enhanced JTextPane
                                                 //for traces and messages
    private JTextField ncyclesValue;
    private Collection refreshables = new ArrayList();
    private SourceLine[] srcLinesPrivate = null;
    private SourceTable sourceTableModel = null;
    private JTable sourceTable = null;
    //private MemoryTableModel memoryTableModel = null;
    private Refreshable regPCSubPanel;

    private int nInst = 0;
    private long nCycles = 0;
    private ArrayList disableOnRUNNING = new ArrayList();
    private ArrayList enableOnRUNNING = new ArrayList();
    private Runnable checkState;
    /** The standard constructor creates a frame.  The frame is populated
     * with:
     *<pre>
     *  +---------------------------------------------------+
     *  |              MENU BAR                             |
     *  +---------------------------------------------------+
     *  +---------------------------------------------------+
     *  |               TOOLBAR                             |
     *  +---------------------------------------------------+
     *  | Info        |  MessagesTraces(top)/Source(bottom) |
     *  |+----------+ |+----------------------------------+ |
     *  ||          | ||                                  | |
     *  || stats    | ||    Trace/Messages                | |
     *  ||          | ||                                  | |
     *  |+----------+ |+----------------------------------+ |
     *  |+----------+ |+----------------------------------+ |
     *  ||          | ||                                  | |
     *  || registers| ||  Source code                     | |
     *  ||          | ||                                  | |
     *  |+----------+ ||                                  | |
     *  |+----------+ ||                                  | |
     *  ||          | ||                                  | |
     *  || memory   | ||                                  | |
     *  ||          | ||                                  | |
     *  |+----------+ |+----------------------------------+ |
     *  +-------------+-------------------------------------+
     *
     *</pre>
     */
    public SwingUI()
    {
	setWarnHandler();
	mem.init(); //I am not happy about this! (!KLUDGE!)
	setConfig();

	// Create the basic frame
	Container contentPane;
	frame = new JFrame("6811 Simulator");
	//	frame.setSize(new Dimension(500, 400)); //Has no effect
	// Set the position of upper left corner
	GraphicsConfiguration gc = frame.getGraphicsConfiguration();
	Rectangle bounds = gc.getBounds();
	frame.setLocation(FRAME_UPPER_X + bounds.x, FRAME_UPPER_Y + bounds.y);

	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	contentPane = frame.getContentPane();
	contentPane.setLayout(new BorderLayout());

	// Create the menu bar
	frame.setJMenuBar(createMenuBar());

	// Create the tool bar
	contentPane.add(createToolBar(), BorderLayout.NORTH);

	// Create the basic panel
	JComponent basicPanel = createBasicPanel();
	contentPane.add(basicPanel, BorderLayout.CENTER);

	//Display the version info in traceMsg
	showMessage(versionToString() + "\n", "legal");

	//Display SIM11_HOME info in traceMsg
	showMessage("SIM11_HOME is: " +
		    (new File(SIM11_HOME)).getAbsolutePath() + "\n");

	//Display basic help
	showMessage("Use the Help -> General button for information " +
		    "about the simulator\n");


	//Do the standard stuff to make the frame appear
	frame.setSize(500,500);
	frame.pack();
	frame.show();
	LCDui lcdUi = new LCDui(0x1400);
	ADCui adcUI = new ADCui();
	ADCControl adcControl = (ADCControl) mem.getCell(0x1030);
	adcControl.setVoltageGetter(adcUI);
	SCIui sciUI = new SCIui();
	new Robot();
    }

 
    /*
     * Many of the following private methods are mere
     * sugar intended to make the basic constructor sequence
     * more understandable and maintainable....
     *
     * Generally, I have tried to organize these private methods
     * hierarchically and sequentially...(but...)
     */

    private void setWarnHandler()
	//GADS!!!! (Oct 10, 2003  mods)
    {
	Warn.setHandler(new Warn.Handler() {
		public void handle(Warn.Warning w)
		{
		    System.out.println("WARNING: " + w.getMessage());
		    JOptionPane pane = new
			JOptionPane(w.getMessage(),
				    JOptionPane.WARNING_MESSAGE);
		    JDialog warning = pane.createDialog(frame, "WARNING: " +
							w.getName());
		    warning.show();
		    System.out.println("Warning handled, total #: "
				       + Warn.getNumWarnings()
				       + " CPU state: "
				       + cpu.getState()
				       );
		    // HARD CODED---stop on WARNING (not pretty!!!)
		    if (cpu.getState() == CPU6811.RUNNING) {
			//cpu.setState(CPU6811.IDLE);
			cpu.stopSim(CPU6811.StopReason.STOPPED_BY_WARNING);
			showMessage("Stopping simulation because of Warning.\n"
				    + " Some warnings may still be active.\n");
		    }
		}
	    });
    }

    private JToolBar createToolBar()
    {
	JButton b;
	JToolBar toolBar = new JToolBar();

	//Basic standalone commands start here.

	////  Single step ("step")
	b = new JButton();
	b.setEnabled(true);
	disableOnRUNNING.add(b);
	toolBar.add(b);
	b.setText("Step");
	//WHAT IS DONE WHEN "STEP" BUTTON IS PUSHED
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    showMessage(stepUI(), "trace");
		    for(Iterator iter = refreshables.iterator();
			iter.hasNext();) {
			((Refreshable)(iter.next())).update();
		    }
		    //Added later...use listeners???
		    sourceTableModel.hilitePCline(cpu.getPC());
		}
	    });
	
	////  Trace ("trace") ... single step as fast as possible ...
	b = new JButton();
	b.setEnabled(true);
	disableOnRUNNING.add(b);
	toolBar.add(b);
	b.setText("Trace(x50)");
	//WHAT IS DONE WHEN "TRACE" BUTTON IS PUSHED
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    showMessage(stepUI(), "trace"); //uncondionally step first
		    for(int i = 0; i < 49; i++) {
			if (cpu.isBreakpoint(cpu.getPC())) {
			    break;
			}
			showMessage(stepUI(), "trace");
			for(Iterator iter = refreshables.iterator();
			    iter.hasNext();) {
			    ((Refreshable)(iter.next())).update();
			}
		    }
		    sourceTableModel.hilitePCline(cpu.getPC());
		}
	    });
	
	////  Run ("run")
	b = new JButton();
	b.setEnabled(true);
	disableOnRUNNING.add(b);
	toolBar.add(b);
	b.setText("Run");
	//WHAT IS DONE WHEN "RUN" BUTTON IS PUSHED
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    nInst = cpu.getNInstructions();
		    nCycles = cpu.getNCycles();
 		    int n = 0;
 		    showMessage("Running at full speed.\n");
		    //		    SwingUtilities.invokeLater(cpu.go(n));
		    ((Thread)cpu.go(n)).start();
		    doComponentEnables();
		    Thread checkState = new Thread(new Runnable() {
			    public void run()
			    {
				cpu.waitNotRunning();
				refreshALL();
				doComponentEnables();
				nInst = cpu.getNInstructions() - nInst;
				nCycles = cpu.getNCycles() - nCycles;
				showMessage("Run done, stopped because: " +
					    cpu.getStopReason() +
					    ", # Instructions: " +
					    nInst +
					    " # Cycles: " +
					    nCycles + "\n");
				
			    }
			});
		    checkState.start();
		}
	    });

	////  Stop ("stop")
	b = new JButton();
	b.setEnabled(false);
	enableOnRUNNING.add(b);
	toolBar.add(b);
	b.setText("Stop");
	//WHAT IS DONE WHEN "STOP" BUTTON IS PUSHED
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    showMessage("Stop pushed\n");
		    cpu.stopSim();
		}
	    });
	
	////  Reset ("reset")
	b = new JButton();
	b.setEnabled(true);
	//enableOnRUNNING.add(b);
	toolBar.add(b);
	b.setText("Reset");
	//WHAT IS DONE WHEN "STOP" BUTTON IS PUSHED
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    showMessage("Reset pushed\n");
		    cpu.reset();
		    refreshALL();

		}
	    });
	
	//Basic commands end here
	
	//Interactive commands start here
	toolBar.add(new javax.swing.JToolBar.Separator());
	//View Points ("vp")
	b = new JButton();
	b.setEnabled(true);
	//	enableOnRUNNING.add(b);
	toolBar.add(b);
	b.setText("Viewpoint");

	//WHAT IS DONE WHEN "Viewpoint" BUTTON IS PUSHED
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    int viewpointAddress;
		    String inputValue = null;
		    try {
		    inputValue =
			JOptionPane.showInputDialog("Viewpoint Address");
		    viewpointAddress =
			Integer.parseInt(inputValue, 16);
		    } catch (NumberFormatException numEx) {
			showMessage("Invalid hex viewpoint address: " +
				    inputValue + ". Ignoring.\n");
			return;
		    }
		    Viewpoint viewpoint = new Viewpoint(viewpointAddress);
		    showMessage("Adding viewpoint at address: " +
				Utility.int16ToHex(viewpointAddress) +
				"\n");
		    viewpoint.show();
		}
	    });
	
	return toolBar;
    }
    
    
    /** Enable/disable components based on current cpu state.
     *
     */
    void doComponentEnables()
    {
	boolean isRunning = false;
	isRunning = cpu.getState() == CPU6811.RUNNING;
	
	for(Iterator iter = disableOnRUNNING.iterator();
	    iter.hasNext();) {
	    JComponent c = (JComponent) iter.next();
	    c.setEnabled(!isRunning);
	}
	
	
	for(Iterator iter = enableOnRUNNING.iterator();
	    iter.hasNext();) {
	    JComponent c = (JComponent) iter.next();
	    c.setEnabled(isRunning);
	}
    }


    /** Refreshes all refreshables. */
    private void refreshALL()
    {
	for(Iterator iter = refreshables.iterator();
	    iter.hasNext();) {
	    ((Refreshable)(iter.next())).update();
	}
	sourceTableModel.hilitePCline(cpu.getPC());
    }

    private JMenuBar createMenuBar()
    {
	JMenuItem menuItem;
	JMenu menu;
	JMenuBar menuBar = new JMenuBar();

	//File menubar
	menuBar.add(menu = new JMenu("File"));
	menuItem = new JMenuItem("Load Source (*.lst)",
				 new ImageIcon(SIM11_HOME +
					       "/images/Open16.gif"));
	menuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    loadSource(".");
		    refreshALL();
		}
	    });
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Unload Source");

	menuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
// 		    String [] fileNames;
// 		    fileNames = getLoadedSrcFileNames();
		    File [] files =  lstLoader.getLoadedSrcFiles();
		    SourceLine [] srcLines;
		    if (files.length > 0) {
			Object[] options = new Object[1 + files.length];
			options[0] = "Cancel";
			
			for(int i = 0; i < files.length; i++) {
			    options[i+1] = files[i].getName();
			}
			
			int choice = JOptionPane.
			    showOptionDialog(frame,
					     "Click to unload a source code "
					     + "file",
					     "Unload source",
					     JOptionPane.YES_NO_CANCEL_OPTION,
					     JOptionPane.QUESTION_MESSAGE,
					     null,
					     options,
					     options[0]);
			if (choice > 0) {
			    srcLines = unload(files[choice-1]);
			    sourceTableModel.addLines(srcLines);
			}
		    }
		}
	    });
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Load Object (*.s19)",
				 new ImageIcon(SIM11_HOME +
					       "/images/Open16.gif"));
	menuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    loadObject(".");
		}
	    });
	menu.add(menuItem);
	
	menu.addSeparator();

	menuItem = new JMenuItem("Samples",
				 new ImageIcon(SIM11_HOME +
					       "/images/Open16.gif"));
	menuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    String root = SIM11_HOME + "/samples";
		    loadSource(root);
		}
	    });
	menu.add(menuItem);

	menu.addSeparator();
	//Lab menu items start here
 	menuItem = new JMenuItem("About labs");
 	menuItem.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e)
 		{
		    new LabInfo();
 		}
 	    });

 	menu.add(menuItem);
	menuItem = new JMenuItem("Labs (object code)",
				 new ImageIcon(SIM11_HOME +
					       "/images/Open16.gif"));
	menuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    String root = SIM11_HOME + "/samples/labs";
		    loadObject(root);
		}
	    });
	menu.add(menuItem);

	menu.addSeparator();

	menuItem = new JMenuItem("Exit");
	menuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    // Could add "are you sure?" dialog, but WHY?
		    System.exit(0);
		}
	    });
	menu.add(menuItem);
	
	//Config menubar
	menuBar.add(menu = new JMenu("Config"));
				
	
 	menuItem = new JMenuItem("Show");
 	menuItem.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e)
 		{
		    new ConfigInfo(configToString());
 		}
 	    });

 	menu.add(menuItem);
 	menu.add(menuItem);

	//Option menubar
	menuBar.add(menu = new JMenu("Options"));
				
	
 	menuItem = new JMenuItem("Load BUFFALO Monitor");
 	menuItem.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e)
 		{
		    String fileFullName = SIM11_HOME + "/samples/mppv96.lst";
		    String fileS19Name = SIM11_HOME + "/samples/mppv96.s19";
		    try {
			srcLinesPrivate = loadLST(fileFullName);
			showMessage("Source file \"" +
				    fileFullName +
				    "\" loaded\n");
			sourceTableModel.addLines(srcLinesPrivate);
			loadS19(fileS19Name);
			
			showMessage("Object file \"" +
				    fileS19Name + 
				    " loaded\n");
			
		    } catch  (Exception exception) {
			String msg = "Unexpected exception loading BUFFALO: " +
			    exception + "\n";
			showMessage(msg);
		    }
		}
		});
	
 	menu.add(menuItem);


	//Help menubar
	menuBar.add(menu = new JMenu("Help"));
				
	menuItem = new JMenuItem("About");
	menuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane pane = new
			JOptionPane(versionToString(),
				    JOptionPane.INFORMATION_MESSAGE);
		    JDialog about = pane.createDialog(frame, "About");
		    about.show();
		}
	    });
	menu.add(menuItem);
	
 	menuItem = new JMenuItem("General");
 	menuItem.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e)
 		{
		    new GeneralInfo();
 		}
 	    });

 	menu.add(menuItem);
 	menuItem = new JMenuItem("License");
 	menuItem.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e)
 		{
		    new LicenseInfo();
 		}
 	    });
 	menu.add(menuItem);
	
	return menuBar;
    }

    private JPanel createBasicPanel()
    {
	LayoutManager layout = new BorderLayout();
	JPanel panel = new JPanel(layout);
	
	// Panel divided West/East (info/traces)
	JPanel genInfo = new JPanel();

	genInfo.setBorder(BorderFactory.createEtchedBorder());
	genInfo.add(createGenInfoPanel());

	panel.add(genInfo, BorderLayout.WEST);
	panel.add(createTracesPanel(), BorderLayout.EAST);
	return panel;
    }

    private JComponent createGenInfoPanel()
    {
	String title;
	JComponent statsPane;
	JComponent registersPane;
	JComponent memoryPane;

	// Create the split pane "stats and registers"
	JSplitPane split = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

	//Populate the statistics (TOP) pane
	statsPane = createStatsPane();
	//	statsPane.setMinimumSize(STATS_MIN);
	statsPane.setPreferredSize(STATS_PREF);
	//	statsPane.setMaximumSize(STATS_MAX);

	title = "Statistics";
	statsPane.setBorder(
			    BorderFactory.createTitledBorder(title));


	//Populate the registers (BOTTOM) pane

	registersPane = createRegistersPane();
	//	registersPane.setMinimumSize(REGS_MIN);
	registersPane.setPreferredSize(REGS_PREF);
	//	registersPane.setMaximumSize(REGS_MAX);
	title = "Registers";
	registersPane.setBorder(
			    BorderFactory.createTitledBorder(title));

	memoryPane = createMemoryPane();
	//	memoryPane.setMinimumSize(REGS_MIN);
	//	memoryPane.setPreferredSize(REGS_PREF);
	//	memoryPane.setMaximumSize(REGS_MAX);
	title = "Memory Views (Editable)";
	memoryPane.setBorder(
			    BorderFactory.createTitledBorder(title));

	//Split pane for registers/memory
	JSplitPane splitBot = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
	splitBot.setTopComponent(registersPane);
	splitBot.setBottomComponent(memoryPane);

	//Populate top/bottom of split pane
	split.setTopComponent(statsPane);
	split.setBottomComponent(splitBot);
	return split;
    }
	
    /** Create the statistics pane.  It is populated
     * with:
     *<pre>
     *  +---------------------------------------------------+
     *  |               statsPane                           |
     *  +---------------------------------------------------+
     *  |+---------------+ |+-----------------------------+ |
     *  || cycles:       | || number of cycles            | |
     *  |+---------------+ |+-----------------------------+ |
     *  |+---------------+ |+-----------------------------+ |
     *  || reads:        | || number of reads             | |
     *  |+---------------+ |+-----------------------------+ |
     *  |+---------------+ |+-----------------------------+ |
     *  || writes:       | || number of writes            | |
     *  |+---------------+ |+-----------------------------+ |
     *  |+---------------+ |+-----------------------------+ |
     *  || instructions: | || number of instructions      | |
     *  |+---------------+ |+-----------------------------+ |
     *  +---------------------------------------------------+
     *
     *</pre>
     */

    private JComponent createStatsPane()
    {
	//	JPanel statsPanel = new JPanel(new GridLayout(4,1));
	JComponent statsPanel = Box.createVerticalBox();
	JPanel statsSubpanel;
	JTextField value;

	//Add cycles subpanel
	statsSubpanel = new StatsPane("# Cycles",
 				     new Refresh() {
 					 public String getValue()
 					 {
 					     return Sim6811.NCyclesToString();
 					 }
 				     });
	refreshables.add((Refreshable) statsSubpanel);
	statsPanel.add(statsSubpanel);

	//Add #reads subpanel
	statsSubpanel = new StatsPane("# Reads",
 				     new Refresh() {
 					 public String getValue()
 					 {
 					     return Sim6811.NReadsToString();
 					 }
 				     });
	refreshables.add((Refreshable) statsSubpanel);
	statsPanel.add(statsSubpanel);

	//Add #writes subpanel
	statsSubpanel = new StatsPane("# Writes",
 				     new Refresh() {
 					 public String getValue()
 					 {
 					     return Sim6811.NWritesToString();
 					 }
 				     });
	refreshables.add((Refreshable) statsSubpanel);
	statsPanel.add(statsSubpanel);

	statsSubpanel = new StatsPane("# Instructions",
				      new Refresh() {
					  public String getValue()
					  {
					      return
					      Sim6811.NInstructionsToString();
					  }
				      });
	refreshables.add((Refreshable) statsSubpanel);
	statsPanel.add(statsSubpanel);

	//Add #warnings subpanel
	statsSubpanel = new StatsPane("# Warnings",
 				     new Refresh() {
 					 public String getValue()
 					 {
					     return
						 Warn.getNumWarningsToString();
 					 }
 				     });
	refreshables.add((Refreshable) statsSubpanel);
	statsPanel.add(statsSubpanel);

	return statsPanel;
    }

    /** Create the registers pane.  It is populated
     * with:
     *<pre>
     *  +---------------------------------------------------+
     *  |               registersPane                       |
     *  +---------------------------------------------------+
     *  |+---------------+ |+-----------------------------+ |
     *  || PC:           | || Program Counter value       | |
     *  |+---------------+ |+-----------------------------+ |
     *  || AccA:         | || Accumulator A value         | |
     *  |+---------------+ |+-----------------------------+ |
     *  +---------------------------------------------------+
     *
     *</pre>
     */

    private JComponent createRegistersPane()
    {
	JComponent registersPanel = Box.createVerticalBox();
	JPanel regSubpanel;
	JTextField value;


	//Add PC subpane
	regSubpanel =
	    new RegisterPane("PC",
			     new Refresh() {
				 public String getValue()
				 {
				     return Sim6811.PCToHexString();
				 }
			     },
			     new ActionListener() {
				 
				 public void actionPerformed(ActionEvent e)
				 {
				     int oldPC = cpu.getPC();
				     JTextField field =
					 (JTextField) (e.getSource());
				     String oldString =
					 Sim6811.PCToHexString();
				     try {
					 int newPC =
				     Integer.parseInt(field.getText(), 16);
					 if (newPC != oldPC) {
					     cpu.setPC(newPC);
					 }
					 sourceTableModel.hilitePCline(newPC);
				     } catch (NumberFormatException fmt) {
					 badHex(oldString, field);
				     }
				 }
			     });
	refreshables.add((Refreshable) regSubpanel);
	regPCSubPanel = (Refreshable) regSubpanel;
	registersPanel.add(regSubpanel);

	//Add AccA subpane
	regSubpanel =
	    new RegisterPane("AccA",
			     new Refresh() {
				 public String getValue()
				 {
				     return Sim6811.AccAToHexString();
				 }
			     },
			     new ActionListener() {
				 
				 public void actionPerformed(ActionEvent e)
				 {
				     int oldValue = cpu.getAccA();
				     JTextField field =
					 (JTextField) (e.getSource());
				     String oldString =
					 Sim6811.AccAToHexString();
				     try {
					 int newValue =
				     Integer.parseInt(field.getText(),16);
					 if (newValue != oldValue) {
					     cpu.setAccA(newValue);
					 }
				     }catch (NumberFormatException fmt) {
					 badHex(oldString, field);
				     }
				 }
			     });
	refreshables.add((Refreshable) regSubpanel);
	registersPanel.add(regSubpanel);

	//Add AccB subpane
	regSubpanel =
	    new RegisterPane("AccB",
			     new Refresh() {
				 public String getValue()
				 {
				     return Sim6811.AccBToHexString();
				 }
			     },
			     new ActionListener() {
				 
				 public void actionPerformed(ActionEvent e)
				 {
				     int oldValue = cpu.getAccB();
				     JTextField field =
					 (JTextField) (e.getSource());
				     String oldString =
					 Sim6811.AccBToHexString();
				     try {
					 int newValue =
				     Integer.parseInt(field.getText(),16);
					 if (newValue != oldValue) {
					     cpu.setAccB(newValue);
					 }
				     }catch (NumberFormatException fmt) {
					 badHex(oldString, field);
				     }
				 }
			     });
	refreshables.add((Refreshable) regSubpanel);
	registersPanel.add(regSubpanel);

	//Add IX subpane
	regSubpanel =
	    new RegisterPane("IX",
			     new Refresh() {
				 public String getValue()
				 {
				     return Sim6811.IXToHexString();
				 }
			     },
			     new ActionListener() {
				 
				 public void actionPerformed(ActionEvent e)
				 {
				     int oldValue = cpu.getIX();
				     JTextField field =
					 (JTextField) (e.getSource());
				     String oldString =
					 Sim6811.IXToHexString();
				     try {
					 int newValue =
				     Integer.parseInt(field.getText(),16);
					 if (newValue != oldValue) {
					     cpu.setIX(newValue);
					 }
				     }catch (NumberFormatException fmt) {
					 badHex(oldString, field);
				     }
				 }
			     });
	refreshables.add((Refreshable) regSubpanel);
	registersPanel.add(regSubpanel);

	//Add IY subpane
	regSubpanel =
	    new RegisterPane("IY",
			     new Refresh() {
				 public String getValue()
				 {
				     return Sim6811.IYToHexString();
				 }
			     },
			     new ActionListener() {
				 
				 public void actionPerformed(ActionEvent e)
				 {
				     int oldValue = cpu.getIY();
				     JTextField field =
					 (JTextField) (e.getSource());
				     String oldString =
					 Sim6811.IYToHexString();
				     try {
					 int newValue =
				     Integer.parseInt(field.getText(),16);
					 if (newValue != oldValue) {
					     cpu.setIY(newValue);
					 }
				     }catch (NumberFormatException fmt) {
					 badHex(oldString, field);
				     }
				 }
			     });
	refreshables.add((Refreshable) regSubpanel);
	registersPanel.add(regSubpanel);

	//Add SP subpane
	regSubpanel =
	    new RegisterPane("SP",
			     new Refresh() {
				 public String getValue()
				 {
				     return Sim6811.SPToHexString();
				 }
			     },
			     new ActionListener() {
				 
				 public void actionPerformed(ActionEvent e)
				 {
				     int oldValue = cpu.getSP();
				     JTextField field =
					 (JTextField) (e.getSource());
				     String oldString =
					 Sim6811.SPToHexString();
				     try {
					 int newValue =
				     Integer.parseInt(field.getText(),16);
					 if (newValue != oldValue) {
					     cpu.setSP(newValue);
					 }
				     }catch (NumberFormatException fmt) {
					 badHex(oldString, field);
				     }
				 }
			     });
	refreshables.add((Refreshable) regSubpanel);
	registersPanel.add(regSubpanel);

	//Add CC subpane
	regSubpanel =
	    new RegisterPane("CC",
			     new Refresh() {
				 public String getValue()
				 {
				     return Sim6811.CCToHexString();
				 }
			     },
			     new ActionListener() {
				 
				 public void actionPerformed(ActionEvent e)
				 {
				     int oldValue = cpu.getCC();
				     JTextField field =
					 (JTextField) (e.getSource());
				     String oldString =
					 Sim6811.CCToHexString();
				     try {
					 int newValue =
				     Integer.parseInt(field.getText(),16);
					 if (newValue != oldValue) {
					     cpu.setCC(newValue);
					 }
				     }catch (NumberFormatException fmt) {
					 badHex(oldString, field);
				     }
				 }
			     });
	refreshables.add((Refreshable) regSubpanel);
	registersPanel.add(regSubpanel);

	registersPanel.add(Box.createGlue());
	return registersPanel;
    }

    /** Create the memory pane.  It is populated
     * with:
     *<pre>
     *  +---------------------------------------------------+
     *  |               memoryPane                          |
     *  +---------------------------------------------------+
     *  |+---------------+ |+-----------------------------+ |
     *  || Address       | || Contents                    | |
     *  |+---------------+ |+-----------------------------+ |
     *  +---------------------------------------------------+
     *
     *</pre>
     */

    private JComponent createMemoryPane()
    {
	Box vbox = Box.createVerticalBox();
	MemoryDumpSwing [] memDump =
	    new MemoryDumpSwing[DEFAULT_MEM_DUMPS.length];
	for(int i = 0; i < DEFAULT_MEM_DUMPS.length; i++) {
	    memDump[i] = new MemoryDumpSwing(DEFAULT_MEM_DUMPS[i]);
	    refreshables.add(memDump[i]);
	    vbox.add(memDump[i]);
	    memDump[i].
		setBorder(BorderFactory.
			  createTitledBorder("Memory Dump " + i));
	}
	return vbox;
    }

    private void badHex(String oldString, JTextField field)
    {
	showMessage("Invalid hex number format IGNORED" +
		    " Restoring to: " +
		    oldString + "\n", "UIwarning");
	field.setText(oldString);
    }

    private JComponent createTracesPanel()
    {
	String title;
	
	// Create the split pane "tracesMessages-Source"
	JSplitPane split = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
	JPanel bottomSource = new JPanel();


	//Populate the trace (TOP) pane
	traceMsg.setEditable(false);

	//Populate top/bottom of split pane
	split.setTopComponent(traceMsg.getScrollable());
	sourceTableModel = new SourceTable();
	sourceTable = new JTable(sourceTableModel);

	sourceTable.setDefaultRenderer(String.class,
				       new StringRenderer(false));//No border

	sourceTable.setShowGrid(false);
	sourceTable.setShowVerticalLines(false);
	TableColumn tableColumn;
	tableColumn = sourceTable.getColumnModel().getColumn(0);//File
	tableColumn.setMinWidth(30);
	tableColumn.setPreferredWidth(60);
	tableColumn.setMaxWidth(100);
	tableColumn = sourceTable.getColumnModel().getColumn(1);//Line number
	tableColumn.setMinWidth(30);
	tableColumn.setPreferredWidth(30);
	tableColumn.setMaxWidth(50);
	tableColumn = sourceTable.getColumnModel().getColumn(2);//PC
	tableColumn.setMinWidth(30);
	tableColumn.setPreferredWidth(30);
	tableColumn.setMaxWidth(50);
	tableColumn = sourceTable.getColumnModel().getColumn(3);//Break
	tableColumn.setMinWidth(20);
	tableColumn.setPreferredWidth(20);
	tableColumn.setMaxWidth(40);
	tableColumn = sourceTable.getColumnModel().getColumn(4);//SRC
	tableColumn.setMinWidth(40);
	//	tableColumn.setPreferredWidth(300);
	tableColumn.setMaxWidth(2000);
        sourceTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	// 	sourceTable.setMinimumSize(SRC_MIN);
	//	sourceTable.setMaximumSize(SRC_MAX);
	//	sourceTable.setPreferredSize(SRC_PREF);
	sourceTable.setPreferredSize( new Dimension(650, 100000));

	ListSelectionModel rowSM = sourceTable.getSelectionModel();
	rowSM.addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    //Ignore extra messages.
                    if (e.getValueIsAdjusting()) return;
                    
                    ListSelectionModel lsm = (ListSelectionModel)e.getSource();
                    if (!lsm.isSelectionEmpty()) {
			int selectedColumn = 
					   sourceTable.getSelectedColumn();
			if (selectedColumn == 3) { //Breakpoint column
			    int oldPC = cpu.getPC();
			    sourceTableModel.hilitePCline(cpu.getPC());
			} else {
			    int selectedRow = lsm.getMinSelectionIndex();
			    SourceTable.ListingLine line =
				(SourceTable.ListingLine)
				sourceTableModel.lines.get(selectedRow);
			    int PC = sourceTableModel.getPCofLine(line);
			    if (PC != -1) {
				cpu.setPC(PC);
				regPCSubPanel.update();
			    }
			}
		    }
		}
	    });
	
	JScrollPane scrollPane =
	    new JScrollPane(sourceTable,
			    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
			    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	//	scrollPane.setMinimumSize(SRC_MIN);
	//	scrollPane.setMaximumSize(SRC_PANEL_MAX);
	scrollPane.setPreferredSize(SRC_PREF);
	
	title = "Source code";
	scrollPane.setBorder(
			    BorderFactory.createTitledBorder(title));
	bottomSource.add(scrollPane);
	split.setBottomComponent(bottomSource);
	return split;
    }

    class StringRenderer extends JLabel
	implements TableCellRenderer {
        Border unselectedBorder = null;
        Border selectedBorder = null;
        boolean isBordered = true;

	static final int FILE_COL = 0;
	static final int PC_COL = 2;
	static final int SRC_COL = 4;

	final Font FILE_FONT = new Font("SansSerif", Font.PLAIN, 12);
	final Font PC_FONT = new Font("SansSerif", Font.PLAIN, 12);
	final Font SRC_FONT = new Font("Monospaced", Font.PLAIN, 12);

        public StringRenderer(boolean isBordered) {
            super();
            this.isBordered = isBordered;
	    this.setForeground(Color.BLACK);
            setOpaque(true); //MUST do this for background to show up.
        }

        public Component getTableCellRendererComponent(
                                JTable table, Object tableCell, 
                                boolean isSelected, boolean hasFocus,
                                int row, int column) {
	    String text = (String) tableCell;
	    if (isSelected) {
		setBackground(table.getSelectionBackground());
	    } else {
		setBackground(Color.WHITE);
	    }
	    
	    switch (column) {
	    case PC_COL:
		setHorizontalAlignment(SwingConstants.RIGHT);
		setFont(PC_FONT);
		break;
	    case FILE_COL:
		setHorizontalAlignment(SwingConstants.LEFT);
		setFont(FILE_FONT);
		break;
	    case SRC_COL:
		setHorizontalAlignment(SwingConstants.LEFT);
		setFont(SRC_FONT);
		break;
	    default://SHould not happen
		assert(false);
		System.exit(1);
	    }
	    setText(text);

            return this;
        }
    }

    /** The data model for the memory dump table.
     *  <p>
     * The fields in the data model are:
     * <ul>
     *  <li>Address</li>
     *  <li>hex contents (x 8).
     * </ul>
     */
    class MemoryTableModel extends AbstractTableModel implements Refreshable {
	final	String[] columnNames = {"Addr", "0/8", "1/9", "2/A", "3/B",
	"4/C", "5/D", "6/E", "7/F"};
	Object[][] data = new Object[0x2000][9];
	MemoryTableModel()
	{
	    super();
	    update();
	    
	}

	public void update()
	{
	    for(int i = 0; i < 0x2000; i++) {
		int contents;
		for(int j = 0; j < 9; j++) {
		    if (j == 0) {
			data[i][j] = Utility.int16ToHex(i*8);
		    } else {
			Cell cell = mem.getCell(i*8 + j - 1);
			if (cell != null) {
			    contents = cell.get();
			    data[i][j] = Utility.int8ToHex(contents);
			} else {
			    data[i][j] = "??";
			}
		    }
		}
	    }
	    fireTableDataChanged();
	}
	
	public int getColumnCount() {
	    return columnNames.length;
	}
	
	public int getRowCount() {
	    return data.length;
	}
	
	public String getColumnName(int col) {
	    return columnNames[col];
	}
	
	public Object getValueAt(int row, int col) {
	    return data[row][col];
	}

	public Class getColumnClass(int c) {
	    return getValueAt(0, c).getClass();
	}
	
	public boolean isCellEditable(int row, int col) {
	    return col != 0;
	}
    }


    /** The data model for the Source Code listing.
     *  <p>
     * The fields in the data model are:
     * <ul>
     *  <li>File name</li>
     *  <li>Line number</li>
     *  <li>Program Counter (if applicable)</li>
     *  <li>Breakpoint notice (if applicable)</li>
     *  <li>Source code</li>
     * </ul>
     */
    class SourceTable extends AbstractTableModel {
	final String[] columnNames = {"File", "Line", "PC", "Brk", "Source"};
	ArrayList lines = new ArrayList();

	/** Populates the table data model.  
	 *
	 */
	public void addLines(SourceLine[] srcLines)
	{
// 	    showMessage("Modifying table model with " +
// 			srcLines.length + " lines.\n");
	    lines = new ArrayList();
	    ListingLine line;
	    for(int i = 0; i < srcLines.length; i++) {
		line = new ListingLine(srcLines[i].file.getName(),
				       srcLines[i].lineNumber,
				       srcLines[i].PC,
				       false,
				       srcLines[i].sourceCode);
		lines.add(line);
	    }
	    fireTableDataChanged();
	}

	/** Highlights the source line corresponding to a
	 * Program Counter value.  If a line is found, it is highlighted
	 * and <code>true</code> is returned; otherwise, no row is
	 * highlighted and <code>false</code> is returned.
	 *
	 * @param pc Program Counter
	 * @return true if line highlighted; else false
	 */

	public boolean hilitePCline(int pc)
	{
	    int row = getRowForPC(pc);
	    //sourceTable.clearSelection();
	    if (row > 0) {
		sourceTable.changeSelection(row, 3, false, false);
	    } else {
		sourceTable.clearSelection();
	    }
	    return row > 0;
	}

	/** Determines the table row number for a Program Counter value.
	 * Returns the (non-negative) row number if found; else -1.
	 */

	private int getRowForPC(int pc)
	{
	    //Step thru ListingLines looking for PC match
	    ListingLine line;
	    int row = 0;
	    boolean found = false;
	    for(Iterator iter = lines.iterator();
		iter.hasNext();
		row++) {
		line = (ListingLine) iter.next();
		//Get PC value of line
		int linePC = getPCofLine(line);
		if (linePC == pc) {
		    found = true;
		    break;
		}
	    }
	    return found ? row : -1;
	}

	int getPCofLine(ListingLine line)
	{
	    String linePCasString = null;
	    int linePC = -1;
	    try {
		linePCasString = (String)line.get(2);
		if (!linePCasString.equals("")) {
		    linePC = Integer.parseInt(linePCasString, 16);
		}
	    } catch (NumberFormatException nfEx) {
		showMessage("NumberFormatException on "
			    + linePCasString
			    + "\n");
	    }
	    return linePC;
	}

	/** The row (line) data structure used in the SourceLine.
	 */
	class ListingLine {
	    private String fileName;
	    private Integer lineNumber;
	    private String PC;
	    private Boolean brk;
	    private String source = null;

	    ListingLine(String fn, int l, int p, boolean b, String s)
	    {
		fileName = fn;
		lineNumber = new Integer(l);
		PC = (p == -1) ? "" : Utility.int16ToHex(p);
		brk = new Boolean(b);
		source = s;
	    }

	    Object get(int col) // KLUDGE TRULY UGLY!!!!
	    {
		assert((col >= 0) && (col <= 4));
	 	switch(col)  {
		case 0: return fileName;
		case 1: return lineNumber;
		case 2: return PC;
		case 3: return brk;
		case 4: return source;
		}
		// should never happen
		System.exit(1);
		return null;
	    }

	    void set(int col, Object element)
	    {
		assert((col >= 0) && (col <= 4));
	 	switch(col)  {
		case 0: fileName = (String) element;
		    return;
		case 1: lineNumber = (Integer) element;
		    return;
		case 2: PC = (String) element;
		    return;
		case 3: brk = (Boolean) element;
		    if (brk == null) {//!KLUDGE!
			showMessage("Cannot set breakpoint here.");
		    } else {
			showMessage("brk: " + element + " PC: " + PC + "\n");
			cpu.toggleBreakpoint(Integer.parseInt(PC, 16));
		    }
		    return;
		case 4: source = (String) element;
		    return;
		}
		// should never happen !NEVER!
		System.exit(1);
	    }
	};
	
	public int getColumnCount() {
            return columnNames.length;
        }
        
        public int getRowCount() {
	    //            return data.length;
	    return lines.size();
        }
	
        public String getColumnName(int col) {
            return columnNames[col];
        }
	
        public Object getValueAt(int row, int col) {
	    Object value = lines.get(row);
	    ListingLine v = (ListingLine) value;
	    Object result = v.get(col);
	    return result;
        }
	
        /*
         * JTable uses this method to determine the default renderer/
         * editor for each cell.  If we didn't implement this method,
         * then the breakpoint column would contain text ("true"/"false"),
         * rather than a check box.
         */
	public Class getColumnClass(int c) {
	    return getValueAt(0, c).getClass();
	}
	
	public boolean isCellEditable(int row, int col) {
	    
	    return (col == 3); //Only breakpoints editable
	}
	
	public void setValueAt(Object value, int row, int col) {
	    if (((ListingLine)lines.get(row)).get(col) instanceof Integer
		&& !(value instanceof Integer)) {                  
                try {
		    ((ListingLine)lines.get(row)).
			set(col, new Integer(value.toString()));
                    fireTableCellUpdated(row, col);
                } catch (NumberFormatException e) {
		    System.exit(1);
                }
            } else {
		ListingLine line = (ListingLine) lines.get(row);
		line.set(col, value);
                lines.set(row, line);
                fireTableCellUpdated(row, col);
            }
        }
    }

    /** Convenience method for default messages.
     *
     * @param msg message to add.
     */
    private void showMessage(String msg)
    {
	showMessage(msg, "message");
    }

    /** Convenience method for styled messages.
     *
     * @param msg message to add.
     * @param style description of style
     */
    private void showMessage(String msg, String style)
    {
	traceMsg.showMessage(msg, style);
    }

    static public void main(String args[])
    {
	if (args.length != 0) {
	    System.err.println("No args allowed yet");
	    System.exit(1);
	}
	SwingUI mainFrame = new SwingUI();
    }

    private void loadObject(String loadPath) {
	JFileChooser chooser = new JFileChooser(loadPath);
	ExampleFileFilter filter = new ExampleFileFilter();
	String fileBaseName = null;
	filter.addExtension("s19");
	filter.setDescription("Object files");
	chooser.setFileFilter(filter);
	int returnVal = chooser.showOpenDialog(null);
	if(returnVal == JFileChooser.APPROVE_OPTION) {
	    File selectedFile = chooser.getSelectedFile();
	    String fileName = selectedFile.getName();
	    String path = selectedFile.getParent()
		+ File.separator;
	    String[] fileNameParts = fileName.split("\\.");
	    fileBaseName = fileNameParts[0];
	    if (fileName.matches(".*\\.s19")) {
		String fileFullName = path +
		    fileBaseName + ".s19";
		try {
		    loadS19(fileFullName);
		    showMessage("Object file \"" +
				fileFullName +
				"\" loaded\n");
		} catch  (IOException exception) {
		    System.err.println(exception);
		    showMessage("Oops! " + fileFullName + "\n");
		}
	    }
	}
    }

    private void loadSource(String loadPath) {
	JFileChooser chooser = new JFileChooser(loadPath);
	ExampleFileFilter filter = new ExampleFileFilter();
	String fileBaseName = null;
	filter.addExtension("asm");
	filter.setDescription("Source files");
	chooser.setFileFilter(filter);
	int returnVal = chooser.showOpenDialog(null);
	if(returnVal == JFileChooser.APPROVE_OPTION) {
	    File selectedFile = chooser.getSelectedFile();
	    String fileName = selectedFile.getName();
	    String path = selectedFile.getParent()
		+ File.separator;
	    String[] fileNameParts = fileName.split("\\.");
	    fileBaseName = fileNameParts[0];
	    if (fileName.matches(".*\\.asm")) {
		// Get and parse the source code listing
		String fileFullName = path +
		    fileBaseName + ".lst";
		try {
		    srcLinesPrivate = loadLST(fileFullName);
		    showMessage("Source file \"" +
				fileFullName +
				"\" loaded\n");
		    sourceTableModel.addLines(srcLinesPrivate);
		    sourceTableModel.hilitePCline(cpu.getPC());
		    loadS19(path + fileBaseName + ".s19");

		    showMessage("Object file \"" +
				path + fileBaseName +
				".s19\" loaded\n");
		    
		} catch  (NoLSTFileException exception) {
		    String msg = "The LST file\n     " +
			exception.getMessage() +
			"\n" +
			"does not exist.\n" +
			" Source code cannot be loaded.";
		    String title = "No LST file";
		    JOptionPane.
			showMessageDialog(frame,
					  msg,
					  title,
					  JOptionPane.
					  WARNING_MESSAGE);
		    
		    showMessage("LST file not found");
		} catch  (NoDuplicateFileException exception) {
		    String msg = "The LST file\n     " +
			exception.getMessage() +
			"\n" +
			"is already loaded.\n" +
			" Source code cannot be re-loaded.";
		    String title = "LST already loaded";
		    JOptionPane.
			showMessageDialog(frame,
					  msg,
					  title,
					  JOptionPane.
					  WARNING_MESSAGE);
		    
		    showMessage("LST file already loaded");
		} catch  (NoS19FileException exception) {
		    String msg = "The S19 file\n     " +
			exception.getMessage() +
			"\n" +
			"does not exist.\n" +
			" Object code cannot be loaded.";
		    String title = "No S19 file";
		    JOptionPane.
			showMessageDialog(frame,
					  msg,
					  title,
					  JOptionPane.
					  WARNING_MESSAGE);
		    
		    showMessage("S19 file not found");
		    
		} catch  (IOException exception) {
		    System.err.println(exception);
		    showMessage("Oops!\n");
		}
		
	    }
	}
    }
}
